@@ -26,10 +26,18 @@
 #include <bonobo.h>
 
 #include "gtkhtml.h"
-#include "persist-stream-impl.h"
+#include "htmlengine-edit.h"
+
 #include "menubar.h"
-#include "toolbar.h"
+#include "persist-file-impl.h"
+#include "persist-stream-impl.h"
 #include "popup.h"
+#include "toolbar.h"
+#include "properties.h"
+#include "text.h"
+#include "paragraph.h"
+#include "link.h"
+#include "body.h"
 
 #include "editor-control-factory.h"
 
@@ -67,7 +75,7 @@ set_frame_cb (BonoboControl *control,
 
 	/* Setup the tool bar.  */
 
-	toolbar = toolbar_setup (uih, control_data);
+	toolbar = toolbar_style (control_data);
 	gtk_box_pack_start (GTK_BOX (control_data->vbox), toolbar, FALSE, FALSE, 0);
 
 	scrolled_window = gtk_scrolled_window_new (NULL, NULL);
@@ -80,30 +88,92 @@ set_frame_cb (BonoboControl *control,
 	/* Setup the menu bar.  */
 
 	menubar_setup (uih, control_data);
+	toolbar_setup (uih, control_data);
+}
+
+static gint
+release (GtkWidget *widget, GdkEventButton *event, GtkHTMLControlData *cd)
+{
+	HTMLEngine *e = cd->html->engine;
+	GtkHTMLEditPropertyType start = GTK_HTML_EDIT_PROPERTY_BODY;
+	gboolean run_dialog = FALSE;
+
+	if (cd->obj) {
+		switch (HTML_OBJECT_TYPE (cd->obj)) {
+		case HTML_TYPE_IMAGE:
+		case HTML_TYPE_LINKTEXTMASTER:
+		case HTML_TYPE_TEXTMASTER:
+			run_dialog = TRUE;
+			break;
+		default:
+		}
+		if (run_dialog) {
+			cd->properties_dialog = gtk_html_edit_properties_dialog_new (cd, FALSE);
+
+			html_cursor_jump_to (e->cursor, e, cd->obj, 0);
+			html_engine_disable_selection (e);
+			html_engine_set_mark (e);
+			html_cursor_jump_to (e->cursor, e, cd->obj, html_object_get_length (cd->obj));
+			html_engine_edit_selection_updater_update_now (e->selection_updater);
+
+			switch (HTML_OBJECT_TYPE (cd->obj)) {
+			case HTML_TYPE_IMAGE:
+				gtk_html_edit_properties_dialog_add_entry (cd->properties_dialog,
+									   GTK_HTML_EDIT_PROPERTY_IMAGE, _("Image"),
+									   image_properties,
+									   image_apply_cb,
+									   image_close_cb);
+				start = GTK_HTML_EDIT_PROPERTY_IMAGE;
+				break;
+			case HTML_TYPE_LINKTEXTMASTER:
+			case HTML_TYPE_TEXTMASTER:
+				gtk_html_edit_properties_dialog_add_entry (cd->properties_dialog,
+									   GTK_HTML_EDIT_PROPERTY_TEXT, _("Text"),
+									   text_properties,
+									   text_apply_cb,
+									   text_close_cb);
+				start = (HTML_OBJECT_TYPE (cd->obj) == HTML_TYPE_TEXTMASTER)
+					? GTK_HTML_EDIT_PROPERTY_TEXT
+					: GTK_HTML_EDIT_PROPERTY_LINK;
+						
+				break;
+			default:
+			}
+			gtk_html_edit_properties_dialog_add_entry (cd->properties_dialog,
+								   GTK_HTML_EDIT_PROPERTY_PARAGRAPH, _("Paragraph"),
+								   paragraph_properties,
+								   paragraph_apply_cb,
+								   paragraph_close_cb);
+			gtk_html_edit_properties_dialog_add_entry (cd->properties_dialog,
+								   GTK_HTML_EDIT_PROPERTY_LINK, _("Link"),
+								   link_properties,
+								   link_apply_cb,
+								   link_close_cb);
+			gtk_html_edit_properties_dialog_add_entry (cd->properties_dialog,
+								   GTK_HTML_EDIT_PROPERTY_BODY, _("Page"),
+								   body_properties,
+								   body_apply_cb,
+								   body_close_cb);
+			gtk_html_edit_properties_dialog_show (cd->properties_dialog);
+			gtk_html_edit_properties_dialog_set_page (cd->properties_dialog, start);
+		}
+	}
+	gtk_signal_disconnect (GTK_OBJECT (widget), cd->releaseId);
+
+	return FALSE;
 }
 
 static gint
 html_button_pressed (GtkWidget *html, GdkEventButton *event, GtkHTMLControlData *cd)
 {
-	HTMLObject *obj;
 	HTMLEngine *engine = cd->html->engine;
 
-	printf ("button pressed %d\n", event->button);
-
-	cd->obj = obj = html_engine_get_object_at (engine,
-						   event->x + engine->x_offset, event->y + engine->y_offset,
-						   NULL, FALSE);	
-
-	if (obj && event->type == GDK_2BUTTON_PRESS && event->button == 1) {
-		switch (HTML_OBJECT_TYPE (obj)) {
-		case HTML_TYPE_IMAGE:
-			image_edit (cd, HTML_IMAGE (obj));
-			break;
-		case HTML_TYPE_LINKTEXTMASTER:
-			link_edit (cd, HTML_LINK_TEXT_MASTER (obj));
-			break;
-		default:
-		}
+	cd->obj = html_engine_get_object_at (engine,
+					     event->x + engine->x_offset,
+					     event->y + engine->y_offset,
+					     NULL, FALSE);
+	if (cd->obj && event->type == GDK_2BUTTON_PRESS && event->button == 1) {
+		cd->releaseId = gtk_signal_connect (GTK_OBJECT (html), "button_release_event", GTK_SIGNAL_FUNC (release), cd);
 	} else if (event->button == 2) {
 		/* pass this for pasting */
 		return TRUE;
@@ -126,6 +196,7 @@ editor_control_factory (BonoboGenericFactory *factory,
 {
 	BonoboControl *control;
 	BonoboPersistStream *stream_impl;
+	BonoboPersistFile *file_impl;
 	GtkWidget *html_widget;
 	GtkWidget *vbox;
 	GtkHTMLControlData *control_data;
@@ -138,6 +209,7 @@ editor_control_factory (BonoboGenericFactory *factory,
 	vbox = gtk_vbox_new (FALSE, 0);
 	gtk_widget_show (vbox);
 
+	/* g_warning ("Creating a new GtkHTML editor control."); */
 	control = bonobo_control_new (vbox);
 
 	/* Bonobo::PersistStream */
@@ -145,7 +217,10 @@ editor_control_factory (BonoboGenericFactory *factory,
 	stream_impl = persist_stream_impl_new (GTK_HTML (html_widget));
 	bonobo_object_add_interface (BONOBO_OBJECT (control), BONOBO_OBJECT (stream_impl));
 
-	g_warning ("Creating a new GtkHTML editor control.");
+	/* Bonobo::PersistFile */
+
+	file_impl = persist_file_impl_new (GTK_HTML (html_widget));
+	bonobo_object_add_interface (BONOBO_OBJECT (control), BONOBO_OBJECT (file_impl));
 
 	/* Part of the initialization must be done after the control is
 	   embedded in its control frame.  We use the "set_frame" signal to
