@@ -19,8 +19,13 @@
     Boston, MA 02111-1307, USA.
 */
 
+#include <config.h>
+
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
+#include <string.h>
+
+#include <gnome.h>
 
 #include "htmlengine-edit-clueflowstyle.h"
 #include "htmlengine-edit-copy.h"
@@ -29,18 +34,27 @@
 #include "htmlengine-edit-fontstyle.h"
 #include "htmlengine-edit-insert.h"
 #include "htmlengine-edit-movement.h"
+#include "htmlengine-edit-cursor.h"
 #include "htmlengine-edit-paste.h"
+#include "htmlengine-edit-text.h"
 #include "htmlengine-edit.h"
 #include "htmlengine-print.h"
+#include "htmlcolorset.h"
 
 #include "gtkhtml-embedded.h"
 #include "gtkhtml-keybinding.h"
 #include "gtkhtml-stream.h"
 #include "gtkhtml-private.h"
+#include <libgnome/gnome-util.h>
 
 
 static GtkLayoutClass *parent_class = NULL;
 
+#ifdef GTKHTML_HAVE_GCONF
+GConfClient *gconf_client = NULL;
+GConfError  *gconf_error  = NULL;
+#endif
+
 enum {
 	TITLE_CHANGED,
 	URL_REQUESTED,
@@ -56,10 +70,23 @@ enum {
 	CURRENT_PARAGRAPH_INDENTATION_CHANGED,
 	CURRENT_PARAGRAPH_ALIGNMENT_CHANGED,
 	INSERTION_FONT_STYLE_CHANGED,
+	INSERTION_COLOR_CHANGED,
+	SIZE_CHANGED,
+	/* keybindings signals */
+	SCROLL,
+	CURSOR_MOVE,
+	INSERT_PARAGRAPH,
+	/* now only last signal */
 	LAST_SIGNAL
 };
 static guint signals [LAST_SIGNAL] = { 0 };
 
+/* keybindings signal hadlers */
+static void scroll              (GtkHTML *html, GtkOrientation orientation, GtkScrollType scroll_type, gfloat position);
+static void cursor_move         (GtkHTML *html, GtkDirectionType dir_type, GtkHTMLCursorSkipType skip);
+static void command             (GtkHTML *html, GtkHTMLCommandType com_type);
+
+static void load_keybindings    (GtkHTMLClass *klass);
 
 /* Values for selection information.  FIXME: what about COMPOUND_STRING and
    TEXT?  */
@@ -207,6 +234,11 @@ update_styles (GtkHTML *html)
 
 	if (html_engine_update_insertion_font_style (engine))
 		gtk_signal_emit (GTK_OBJECT (html), signals[INSERTION_FONT_STYLE_CHANGED], engine->insertion_font_style);
+
+	if (html_engine_update_insertion_color (engine))
+		gtk_signal_emit (GTK_OBJECT (html), signals[INSERTION_COLOR_CHANGED], engine->insertion_color);
+	/* TODO add insertion_url_or_targed_changed signal */
+	html_engine_update_insertion_url_and_target (engine);
 }
 
 
@@ -533,29 +565,50 @@ destroy (GtkObject *object)
 
 
 /* GtkWidget methods.  */
+static void
+style_set (GtkWidget *widget,
+	   GtkStyle  *previous_style)
+{
+	HTMLEngine *engine = GTK_HTML (widget)->engine;
+	
+	html_colorset_set_style (engine->defaultSettings->color_set,
+				 widget->style);
+	html_colorset_set_unchanged (engine->settings->color_set,
+				     engine->defaultSettings->color_set);
+	html_engine_schedule_update (engine);
+}
 
 static gint
 key_press_event (GtkWidget *widget,
 		 GdkEventKey *event)
 {
-	GtkHTML *html;
-	HTMLEngine *engine;
+	GtkHTML *html = GTK_HTML (widget);
 	gboolean retval;
-	gboolean do_update_styles;
+	gint position = html->engine->cursor->position;
 
-	html = GTK_HTML (widget);
-	engine = html->engine;
+	html->binding_handled = FALSE;
+	gtk_bindings_activate (GTK_OBJECT (widget), event->keyval, event->state);
+	retval = html->binding_handled;
 
-	if (! html_engine_get_editable (engine)) {
-		/* FIXME handle differently in this case */
-		return FALSE;
+
+	if (! retval
+	    && html_engine_get_editable (html->engine)
+	    && ! (event->state & (GDK_CONTROL_MASK | GDK_MOD1_MASK))
+	    && event->length > 0) {
+		html_engine_delete_selection (html->engine, TRUE);
+		html_engine_insert (html->engine, event->string, event->length);
+		retval = TRUE;
 	}
 
-	retval = gtk_html_handle_key_event (GTK_HTML (widget), event, &do_update_styles);
+	if (!retval && GTK_WIDGET_CLASS (parent_class)->key_press_event)
+		retval = GTK_WIDGET_CLASS (parent_class)->key_press_event (widget, event);
 
-	if (retval == TRUE) {
+	if (retval && html_engine_get_editable (html->engine))
+		html_engine_reset_blinking_cursor (html->engine);
+
+	if (retval) {
 		queue_draw (html);
-		if (do_update_styles)
+		if (position != html->engine->cursor->position)
 			update_styles (html);
 	}
 
@@ -575,6 +628,7 @@ realize (GtkWidget *widget)
 	if (GTK_WIDGET_CLASS (parent_class)->realize)
 		(* GTK_WIDGET_CLASS (parent_class)->realize) (widget);
 
+	widget->style = gtk_style_attach (widget->style, widget->window);
 	gdk_window_set_events (html->layout.bin_window,
 			       (gdk_window_get_events (html->layout.bin_window)
 				| GDK_EXPOSURE_MASK | GDK_POINTER_MOTION_MASK
@@ -815,7 +869,7 @@ button_press_event (GtkWidget *widget,
 		update_styles (html);
 	}
 
-	if (html->allow_selection) {
+	if (html->allow_selection && !(event->state & GDK_SHIFT_MASK)) {
 		gtk_grab_add (widget);
 		gdk_pointer_grab (widget->window, TRUE,
 				  (GDK_BUTTON_RELEASE_MASK
@@ -829,7 +883,13 @@ button_press_event (GtkWidget *widget,
 
 	html->button_pressed = TRUE;
 
-	html_engine_disable_selection (engine);
+	if (!(event->state & GDK_SHIFT_MASK))
+		html_engine_disable_selection (engine);
+	else
+		html_engine_select_region (engine,
+					   html->selection_x1, html->selection_y1,
+					   event->x + engine->x_offset, event->y + engine->y_offset,
+					   TRUE);
 
 	return TRUE;
 }
@@ -970,7 +1030,7 @@ selection_received (GtkWidget *widget,
 		printf ("selection text \"%.*s\"\n",
 			selection_data->length, selection_data->data); 
 
-		html_engine_disable_selection (GTK_HTML (widget)->engine);
+		html_engine_delete_selection (GTK_HTML (widget)->engine, TRUE);
 		html_engine_insert (GTK_HTML (widget)->engine, 
 				    selection_data->data,
 				    selection_data->length);
@@ -1028,10 +1088,66 @@ set_adjustments (GtkLayout     *layout,
 
 /* Initialization.  */
 
+#ifdef GTKHTML_HAVE_GCONF
+
+static void
+client_notify (GConfClient* client,
+	       guint cnxn_id,
+	       const gchar* key,
+	       GConfValue* value,
+	       gboolean is_default,
+	       gpointer user_data)
+{
+	GtkHTMLClass *klass = (GtkHTMLClass *) user_data;
+	GtkHTMLClassProperties *prop = klass->properties;	
+	gchar *tkey;
+
+	g_assert (client == gconf_client);
+
+	printf ("client notify key %s\n", key);
+
+	g_assert (key);
+	tkey = strrchr (key, '/');
+	g_assert (tkey);
+
+	if (!strcmp (tkey, "/magic_links")) {
+		prop->magic_links = gconf_client_get_bool (client, key, NULL);
+	} else if (!strcmp (tkey, "/keybindings_theme")) {
+		g_free (prop->keybindings_theme);
+		prop->keybindings_theme = gconf_client_get_string (client, key, NULL);
+		load_keybindings (klass);
+	} else
+		g_assert_not_reached ();
+}
+
+#endif
+
+static void
+init_properties (GtkHTMLClass *klass)
+{
+	klass->properties = gtk_html_class_properties_new ();
+#ifdef GTKHTML_HAVE_GCONF
+	if (gconf_is_initialized ()) {
+		gconf_client = gconf_client_new ();
+		if (!gconf_client)
+			g_error ("cannot create gconf_client\n");
+		gconf_client_add_dir (gconf_client, GTK_HTML_GCONF_DIR, GCONF_CLIENT_PRELOAD_ONELEVEL, &gconf_error);
+		if (gconf_error)
+			g_error ("gconf error: %s\n", gconf_error->str);
+		gtk_html_class_properties_load (klass->properties, gconf_client);
+	} else
+		g_error ("gconf is not initialized, please call gconf_init before using GtkHTML library\n");
+#endif
+	load_keybindings (klass);
+#ifdef GTKHTML_HAVE_GCONF
+	gconf_client_notify_add (gconf_client, GTK_HTML_GCONF_DIR, client_notify, klass, NULL, NULL);
+#endif
+}
+
 static void
 class_init (GtkHTMLClass *klass)
 {
-	GtkHTMLClass *html_class;
+	GtkHTMLClass   *html_class;
 	GtkWidgetClass *widget_class;
 	GtkObjectClass *object_class;
 	GtkLayoutClass *layout_class;
@@ -1040,7 +1156,7 @@ class_init (GtkHTMLClass *klass)
 	widget_class = (GtkWidgetClass *)klass;
 	object_class = (GtkObjectClass *)klass;
 	layout_class = (GtkLayoutClass *)klass;
-	
+
 	object_class->destroy = destroy;
 
 	parent_class = gtk_type_class (GTK_TYPE_LAYOUT);
@@ -1169,12 +1285,57 @@ class_init (GtkHTMLClass *klass)
 				GTK_TYPE_NONE, 1,
 				GTK_TYPE_INT);
 	
+	signals [INSERTION_COLOR_CHANGED] =
+		gtk_signal_new ("insertion_color_changed",
+				GTK_RUN_FIRST,
+				object_class->type,
+				GTK_SIGNAL_OFFSET (GtkHTMLClass, insertion_color_changed),
+				gtk_marshal_NONE__POINTER,
+				GTK_TYPE_NONE, 1,
+				GTK_TYPE_POINTER);
+	
+	signals [SIZE_CHANGED] = 
+		gtk_signal_new ("size_changed",
+				GTK_RUN_FIRST,
+				object_class->type,
+				GTK_SIGNAL_OFFSET (GtkHTMLClass, size_changed),
+				gtk_marshal_NONE__NONE,
+				GTK_TYPE_NONE, 0);
+
+	/* signals for keybindings */
+	signals [SCROLL] =
+		gtk_signal_new ("scroll",
+				GTK_RUN_LAST | GTK_RUN_ACTION,
+				object_class->type,
+				GTK_SIGNAL_OFFSET (GtkHTMLClass, scroll),
+				gtk_marshal_NONE__ENUM_FLOAT,
+				GTK_TYPE_NONE, 3,
+				GTK_TYPE_ORIENTATION,
+				GTK_TYPE_SCROLL_TYPE, GTK_TYPE_FLOAT);
+
+	signals [CURSOR_MOVE] =
+		gtk_signal_new ("cursor_move",
+				GTK_RUN_LAST | GTK_RUN_ACTION,
+				object_class->type,
+				GTK_SIGNAL_OFFSET (GtkHTMLClass, cursor_move),
+				gtk_marshal_NONE__INT_INT,
+				GTK_TYPE_NONE, 2, GTK_TYPE_DIRECTION_TYPE, GTK_TYPE_HTML_CURSOR_SKIP);
+
+	signals [INSERT_PARAGRAPH] =
+		gtk_signal_new ("command",
+				GTK_RUN_LAST | GTK_RUN_ACTION,
+				object_class->type,
+				GTK_SIGNAL_OFFSET (GtkHTMLClass, command),
+				gtk_marshal_NONE__ENUM,
+				GTK_TYPE_NONE, 1, GTK_TYPE_HTML_COMMAND);
+
 	gtk_object_class_add_signals (object_class, signals, LAST_SIGNAL);
 
 	object_class->destroy = destroy;
 	
 	widget_class->realize = realize;
 	widget_class->unrealize = unrealize;
+	widget_class->style_set = style_set;
 	widget_class->draw = draw;
 	widget_class->key_press_event = key_press_event;
 	widget_class->expose_event  = expose;
@@ -1189,6 +1350,12 @@ class_init (GtkHTMLClass *klass)
 	widget_class->selection_clear_event = selection_clear_event;
 
 	layout_class->set_scroll_adjustments = set_adjustments;
+
+	html_class->scroll            = scroll;
+	html_class->cursor_move       = cursor_move;
+	html_class->command           = command;
+
+	init_properties (klass);
 }
 
 static void
@@ -1280,8 +1447,7 @@ gtk_html_construct (GtkWidget *htmlw)
 
 	html = GTK_HTML (htmlw);
 
-	html->engine = html_engine_new ();
-	html->engine->widget = html; /* FIXME FIXME */
+	html->engine = html_engine_new (htmlw);
 
 	gtk_signal_connect (GTK_OBJECT (html->engine), "title_changed",
 			    GTK_SIGNAL_FUNC (html_engine_title_changed_cb), html);
@@ -1331,6 +1497,8 @@ gtk_html_begin (GtkHTML *html)
 {
 	GtkHTMLStream *handle;
 
+	g_return_val_if_fail (! gtk_html_get_editable (html), NULL);
+
 	handle = html_engine_begin (html->engine);
 	if (handle == NULL)
 		return NULL;
@@ -1445,8 +1613,10 @@ gtk_html_private_calc_scrollbars (GtkHTML *html)
 	hadj->step_increment = 14; /* FIXME */
 	hadj->page_increment = html->engine->width;
 
-	if (width != layout->width || height != layout->height)
+	if ((width != layout->width) || (height != layout->height)) {
+		gtk_signal_emit (GTK_OBJECT (html), signals[SIZE_CHANGED]);
 		gtk_layout_set_size (layout, width, height);
+	}
 }
 
 
@@ -1458,6 +1628,9 @@ gtk_html_set_editable (GtkHTML *html,
 	g_return_if_fail (GTK_IS_HTML (html));
 
 	html_engine_set_editable (html->engine, editable);
+
+	if (editable)
+		update_styles (html);
 }
 
 gboolean
@@ -1522,6 +1695,12 @@ gtk_html_set_paragraph_style (GtkHTML *html,
 			 style);
 }
 
+GtkHTMLParagraphStyle
+gtk_html_get_paragraph_style (GtkHTML *html)
+{
+	return html_engine_get_current_clueflow_style (html->engine);
+}
+
 void
 gtk_html_indent (GtkHTML *html,
 		 gint delta)
@@ -1543,11 +1722,31 @@ gtk_html_set_font_style (GtkHTML *html,
 	g_return_if_fail (html != NULL);
 	g_return_if_fail (GTK_IS_HTML (html));
 
-	html_engine_set_font_style (html->engine, and_mask, or_mask);
+	if (html_engine_set_font_style (html->engine, and_mask, or_mask))
+		gtk_signal_emit (GTK_OBJECT (html), signals [INSERTION_FONT_STYLE_CHANGED],
+				 html->engine->insertion_font_style);
 }
 
 void
-gtk_html_align_paragraph (GtkHTML *html,
+gtk_html_toggle_font_style (GtkHTML *html,
+			    GtkHTMLFontStyle style)
+{
+	g_return_if_fail (html != NULL);
+	g_return_if_fail (GTK_IS_HTML (html));
+
+	if (html_engine_toggle_font_style (html->engine, style))
+		gtk_signal_emit (GTK_OBJECT (html), signals [INSERTION_FONT_STYLE_CHANGED],
+				 html->engine->insertion_font_style);
+}
+
+GtkHTMLParagraphAlignment
+gtk_html_get_paragraph_alignment (GtkHTML *html)
+{
+	return paragraph_alignment_to_html (html_engine_get_current_clueflow_alignment (html->engine));
+}
+
+void
+gtk_html_set_paragraph_alignment (GtkHTML *html,
 			  GtkHTMLParagraphAlignment alignment)
 {
 	HTMLHAlignType align;
@@ -1617,5 +1816,543 @@ gtk_html_redo (GtkHTML *html)
 void
 gtk_html_set_default_background_color (GtkHTML *html, GdkColor *c)
 {
-	html_settings_set_color (html->engine->defaultSettings, HTMLBgColor, c);
+	html_colorset_set_color (html->engine->defaultSettings->color_set, c, HTMLBgColor);
+}
+
+/*******************************************
+
+   keybindings
+
+*/
+
+static void
+scroll (GtkHTML *html,
+	GtkOrientation orientation,
+	GtkScrollType  scroll_type,
+	gfloat         position)
+{
+	GtkAdjustment *adj;
+	gfloat delta;
+
+	/* we dont want scroll in editable (move cursor instead) */
+	if (html_engine_get_editable (html->engine))
+		return;
+
+	adj = (orientation == GTK_ORIENTATION_VERTICAL)
+		? gtk_layout_get_vadjustment (GTK_LAYOUT (html)) : gtk_layout_get_hadjustment (GTK_LAYOUT (html));
+
+
+	switch (scroll_type) {
+	case GTK_SCROLL_STEP_FORWARD:
+		delta = adj->step_increment;
+		break;
+	case GTK_SCROLL_STEP_BACKWARD:
+		delta = -adj->step_increment;
+		break;
+	case GTK_SCROLL_PAGE_FORWARD:
+		delta = adj->page_increment;
+		break;
+	case GTK_SCROLL_PAGE_BACKWARD:
+		delta = -adj->page_increment;
+		break;
+	default:
+		delta = 0.0;
+		g_assert_not_reached ();
+	}
+
+	adj->value = CLAMP (adj->value + delta, adj->lower, adj->upper - adj->page_size);
+	gtk_adjustment_value_changed (adj);
+
+	html->binding_handled = TRUE;
+}
+
+static void
+scroll_by_amount (GtkHTML *html,
+		  gint amount)
+{
+	GtkLayout *layout;
+	GtkAdjustment *adj;
+	gfloat new_value;
+
+	layout = GTK_LAYOUT (html);
+	adj = layout->vadjustment;
+
+	new_value = adj->value + (gfloat) amount;
+	if (new_value < adj->lower)
+		new_value = adj->lower;
+	else if (new_value > adj->upper)
+		new_value = adj->upper;
+
+	gtk_adjustment_set_value (adj, new_value);
+}
+
+static void
+cursor_move (GtkHTML *html, GtkDirectionType dir_type, GtkHTMLCursorSkipType skip)
+{
+	gint amount;
+
+	if (!html_engine_get_editable (html->engine))
+		return;
+
+	if (html->engine->shift_selection)
+		html_engine_disable_selection (html->engine);
+
+	switch (skip) {
+	case GTK_HTML_CURSOR_SKIP_ONE:
+		switch (dir_type) {
+		case GTK_DIR_LEFT:
+			html_engine_move_cursor (html->engine, HTML_ENGINE_CURSOR_LEFT, 1);
+			break;
+		case GTK_DIR_RIGHT:
+			html_engine_move_cursor (html->engine, HTML_ENGINE_CURSOR_RIGHT, 1);
+			break;
+		case GTK_DIR_UP:
+			html_engine_move_cursor (html->engine, HTML_ENGINE_CURSOR_UP, 1);
+			break;
+		case GTK_DIR_DOWN:
+			html_engine_move_cursor (html->engine, HTML_ENGINE_CURSOR_DOWN, 1);
+			break;
+		default:
+			g_assert_not_reached ();
+		}
+		break;
+	case GTK_HTML_CURSOR_SKIP_WORD:
+		switch (dir_type) {
+		case GTK_DIR_UP:
+		case GTK_DIR_LEFT:
+			html_engine_backward_word (html->engine);
+			break;
+		case GTK_DIR_DOWN:
+		case GTK_DIR_RIGHT:
+			html_engine_forward_word (html->engine);
+			break;
+		default:
+			g_assert_not_reached ();
+		}
+		break;
+	case GTK_HTML_CURSOR_SKIP_PAGE:
+		switch (dir_type) {
+		case GTK_DIR_UP:
+		case GTK_DIR_LEFT:
+			if ((amount = html_engine_scroll_up (html->engine, GTK_WIDGET (html)->allocation.height)) > 0)
+				scroll_by_amount (html, - amount);
+			break;
+		case GTK_DIR_DOWN:
+		case GTK_DIR_RIGHT:
+			if ((amount = html_engine_scroll_down (html->engine, GTK_WIDGET (html)->allocation.height)) > 0)
+				scroll_by_amount (html, amount);
+			break;
+		default:
+			g_assert_not_reached ();
+		}
+		break;
+	case GTK_HTML_CURSOR_SKIP_ALL:
+		switch (dir_type) {
+		case GTK_DIR_LEFT:
+			html_engine_beginning_of_line (html->engine);
+			break;
+		case GTK_DIR_RIGHT:
+			html_engine_end_of_line (html->engine);
+			break;
+		case GTK_DIR_UP:
+			html_engine_beginning_of_document (html->engine);
+			break;
+		case GTK_DIR_DOWN:
+			html_engine_end_of_document (html->engine);
+			break;
+		default:
+			g_assert_not_reached ();
+		}
+		break;
+	default:
+		g_assert_not_reached ();
+	}
+
+	html->binding_handled = TRUE;
+}
+
+static void
+move_selection (GtkHTML *html, GtkHTMLCommandType com_type)
+{
+	gint amount;
+
+	if (!html_engine_get_editable (html->engine))
+		return;
+
+	html->engine->shift_selection = TRUE;
+	if (!html->engine->mark)
+		html_engine_set_mark (html->engine);
+	switch (com_type) {
+	case GTK_HTML_COMMAND_MODIFY_SELECTION_UP:
+		html_engine_move_cursor (html->engine, HTML_ENGINE_CURSOR_UP, 1);
+		break;
+	case GTK_HTML_COMMAND_MODIFY_SELECTION_DOWN:
+		html_engine_move_cursor (html->engine, HTML_ENGINE_CURSOR_DOWN, 1);
+		break;
+	case GTK_HTML_COMMAND_MODIFY_SELECTION_LEFT:
+		html_engine_move_cursor (html->engine, HTML_ENGINE_CURSOR_LEFT, 1);
+		break;
+	case GTK_HTML_COMMAND_MODIFY_SELECTION_RIGHT:
+		html_engine_move_cursor (html->engine, HTML_ENGINE_CURSOR_RIGHT, 1);
+		break;
+	case GTK_HTML_COMMAND_MODIFY_SELECTION_BOL:
+		html_engine_beginning_of_line (html->engine);
+		break;
+	case GTK_HTML_COMMAND_MODIFY_SELECTION_EOL:
+		html_engine_end_of_line (html->engine);
+		break;
+	case GTK_HTML_COMMAND_MODIFY_SELECTION_BOD:
+		html_engine_beginning_of_document (html->engine);
+		break;
+	case GTK_HTML_COMMAND_MODIFY_SELECTION_EOD:
+		html_engine_end_of_document (html->engine);
+		break;
+	case GTK_HTML_COMMAND_MODIFY_SELECTION_PAGEUP:
+		if ((amount = html_engine_scroll_up (html->engine, GTK_WIDGET (html)->allocation.height)) > 0)
+			scroll_by_amount (html, - amount);
+		break;
+	case GTK_HTML_COMMAND_MODIFY_SELECTION_PAGEDOWN:
+		if ((amount = html_engine_scroll_down (html->engine, GTK_WIDGET (html)->allocation.height)) > 0)
+			scroll_by_amount (html, amount);
+		break;
+	default:
+		g_assert_not_reached ();
+	}
+}
+
+static void
+command (GtkHTML *html, GtkHTMLCommandType com_type)
+{
+	if (!html_engine_get_editable (html->engine))
+		return;
+
+	/* printf ("command %d\n", com_type); */
+
+	switch (com_type) {
+	case GTK_HTML_COMMAND_UNDO:
+		html_engine_undo (html->engine);
+		break;
+	case GTK_HTML_COMMAND_REDO:
+		html_engine_redo (html->engine);
+		break;
+	case GTK_HTML_COMMAND_COPY:
+		if (html->engine->active_selection)
+			html_engine_copy (html->engine);
+		break;
+	case GTK_HTML_COMMAND_CUT:
+		html_engine_cut (html->engine, TRUE);
+		break;
+	case GTK_HTML_COMMAND_CUT_LINE:
+		html_engine_cut_line (html->engine, TRUE);
+		break;
+	case GTK_HTML_COMMAND_PASTE:
+		if (html->engine->cut_buffer)
+			html_engine_paste (html->engine, TRUE);
+		break;
+	case GTK_HTML_COMMAND_INSERT_PARAGRAPH:
+		html_engine_delete_selection (html->engine, TRUE);
+		html_engine_insert (html->engine, "\n", 1);
+		break;
+	case GTK_HTML_COMMAND_DELETE:
+		if (html->engine->mark != NULL
+		    && html->engine->mark->position != html->engine->cursor->position)
+			html_engine_delete_selection (html->engine, TRUE);
+		else
+			html_engine_delete (html->engine, 1, TRUE, FALSE);
+		break;
+	case GTK_HTML_COMMAND_DELETE_BACK:
+		if (html->engine->mark != NULL
+		    && html->engine->mark->position != html->engine->cursor->position)
+			html_engine_delete_selection (html->engine, TRUE);
+		else
+			html_engine_delete (html->engine, 1, TRUE, TRUE);
+		break;
+	case GTK_HTML_COMMAND_SET_MARK:
+		html_engine_set_mark (html->engine);
+		break;
+	case GTK_HTML_COMMAND_DISABLE_SELECTION:
+		html_engine_disable_selection (html->engine);
+		break;
+	case GTK_HTML_COMMAND_TOGGLE_BOLD:
+		gtk_html_toggle_font_style (html, GTK_HTML_FONT_STYLE_BOLD);
+		break;
+	case GTK_HTML_COMMAND_TOGGLE_ITALIC:
+		gtk_html_toggle_font_style (html, GTK_HTML_FONT_STYLE_ITALIC);
+		break;
+	case GTK_HTML_COMMAND_TOGGLE_UNDERLINE:
+		gtk_html_toggle_font_style (html, GTK_HTML_FONT_STYLE_UNDERLINE);
+		break;
+	case GTK_HTML_COMMAND_TOGGLE_STRIKEOUT:
+		gtk_html_toggle_font_style (html, GTK_HTML_FONT_STYLE_STRIKEOUT);
+		break;
+	case GTK_HTML_COMMAND_SIZE_MINUS_2:
+		gtk_html_set_font_style (html, ~GTK_HTML_FONT_STYLE_SIZE_MASK, GTK_HTML_FONT_STYLE_SIZE_1);
+		break;
+	case GTK_HTML_COMMAND_SIZE_MINUS_1:
+		gtk_html_set_font_style (html, ~GTK_HTML_FONT_STYLE_SIZE_MASK, GTK_HTML_FONT_STYLE_SIZE_2);
+		break;
+	case GTK_HTML_COMMAND_SIZE_PLUS_0:
+		gtk_html_set_font_style (html, ~GTK_HTML_FONT_STYLE_SIZE_MASK, GTK_HTML_FONT_STYLE_SIZE_3);
+		break;
+	case GTK_HTML_COMMAND_SIZE_PLUS_1:
+		gtk_html_set_font_style (html, ~GTK_HTML_FONT_STYLE_SIZE_MASK, GTK_HTML_FONT_STYLE_SIZE_4);
+		break;
+	case GTK_HTML_COMMAND_SIZE_PLUS_2:
+		gtk_html_set_font_style (html, ~GTK_HTML_FONT_STYLE_SIZE_MASK, GTK_HTML_FONT_STYLE_SIZE_5);
+		break;
+	case GTK_HTML_COMMAND_SIZE_PLUS_3:
+		gtk_html_set_font_style (html, ~GTK_HTML_FONT_STYLE_SIZE_MASK, GTK_HTML_FONT_STYLE_SIZE_6);
+		break;
+	case GTK_HTML_COMMAND_SIZE_PLUS_4:
+		gtk_html_set_font_style (html, ~GTK_HTML_FONT_STYLE_SIZE_MASK, GTK_HTML_FONT_STYLE_SIZE_7);
+		break;
+	case GTK_HTML_COMMAND_SIZE_INCREASE:
+		html_engine_font_size_inc_dec (html->engine, TRUE);
+		break;
+	case GTK_HTML_COMMAND_SIZE_DECREASE:
+		html_engine_font_size_inc_dec (html->engine, FALSE);
+		break;
+	case GTK_HTML_COMMAND_ALIGN_LEFT:
+		gtk_html_set_paragraph_alignment (html, GTK_HTML_PARAGRAPH_ALIGNMENT_LEFT);
+		break;
+	case GTK_HTML_COMMAND_ALIGN_CENTER:
+		gtk_html_set_paragraph_alignment (html, GTK_HTML_PARAGRAPH_ALIGNMENT_CENTER);
+		break;
+	case GTK_HTML_COMMAND_ALIGN_RIGHT:
+		gtk_html_set_paragraph_alignment (html, GTK_HTML_PARAGRAPH_ALIGNMENT_RIGHT);
+		break;
+	case GTK_HTML_COMMAND_INDENT_INC:
+		gtk_html_indent (html, +1);
+		break;
+	case GTK_HTML_COMMAND_INDENT_DEC:
+		gtk_html_indent (html, -1);
+		break;
+	case GTK_HTML_COMMAND_PARAGRAPH_STYLE_NORMAL:
+		gtk_html_set_paragraph_style (html, GTK_HTML_PARAGRAPH_STYLE_NORMAL);
+		break;
+	case GTK_HTML_COMMAND_PARAGRAPH_STYLE_H1:
+		gtk_html_set_paragraph_style (html, GTK_HTML_PARAGRAPH_STYLE_H1);
+		break;
+	case GTK_HTML_COMMAND_PARAGRAPH_STYLE_H2:
+		gtk_html_set_paragraph_style (html, GTK_HTML_PARAGRAPH_STYLE_H2);
+		break;
+	case GTK_HTML_COMMAND_PARAGRAPH_STYLE_H3:
+		gtk_html_set_paragraph_style (html, GTK_HTML_PARAGRAPH_STYLE_H3);
+		break;
+	case GTK_HTML_COMMAND_PARAGRAPH_STYLE_H4:
+		gtk_html_set_paragraph_style (html, GTK_HTML_PARAGRAPH_STYLE_H4);
+		break;
+	case GTK_HTML_COMMAND_PARAGRAPH_STYLE_H5:
+		gtk_html_set_paragraph_style (html, GTK_HTML_PARAGRAPH_STYLE_H5);
+		break;
+	case GTK_HTML_COMMAND_PARAGRAPH_STYLE_H6:
+		gtk_html_set_paragraph_style (html, GTK_HTML_PARAGRAPH_STYLE_H6);
+		break;
+	case GTK_HTML_COMMAND_PARAGRAPH_STYLE_PRE:
+		gtk_html_set_paragraph_style (html, GTK_HTML_PARAGRAPH_STYLE_PRE);
+		break;
+	case GTK_HTML_COMMAND_PARAGRAPH_STYLE_ADDRESS:
+		gtk_html_set_paragraph_style (html, GTK_HTML_PARAGRAPH_STYLE_ADDRESS);
+		break;
+	case GTK_HTML_COMMAND_PARAGRAPH_STYLE_ITEMDOTTED:
+		gtk_html_set_paragraph_style (html, GTK_HTML_PARAGRAPH_STYLE_ITEMDOTTED);
+		break;
+	case GTK_HTML_COMMAND_PARAGRAPH_STYLE_ITEMROMAN:
+		gtk_html_set_paragraph_style (html, GTK_HTML_PARAGRAPH_STYLE_ITEMROMAN);
+		break;
+	case GTK_HTML_COMMAND_PARAGRAPH_STYLE_ITEMDIGIT:
+		gtk_html_set_paragraph_style (html, GTK_HTML_PARAGRAPH_STYLE_ITEMDIGIT);
+		break;
+	case GTK_HTML_COMMAND_MODIFY_SELECTION_UP:
+	case GTK_HTML_COMMAND_MODIFY_SELECTION_DOWN:
+	case GTK_HTML_COMMAND_MODIFY_SELECTION_LEFT:
+	case GTK_HTML_COMMAND_MODIFY_SELECTION_RIGHT:
+	case GTK_HTML_COMMAND_MODIFY_SELECTION_BOL:
+	case GTK_HTML_COMMAND_MODIFY_SELECTION_EOL:
+	case GTK_HTML_COMMAND_MODIFY_SELECTION_BOD:
+	case GTK_HTML_COMMAND_MODIFY_SELECTION_EOD:
+	case GTK_HTML_COMMAND_MODIFY_SELECTION_PAGEUP:
+	case GTK_HTML_COMMAND_MODIFY_SELECTION_PAGEDOWN:
+		move_selection (html, com_type);
+		break;
+	case GTK_HTML_COMMAND_CAPITALIZE_WORD:
+		html_engine_capitalize_word (html->engine);
+		break;
+	case GTK_HTML_COMMAND_UPCASE_WORD:
+		html_engine_upcase_downcase_word (html->engine, TRUE);
+		break;
+	case GTK_HTML_COMMAND_DOWNCASE_WORD:
+		html_engine_upcase_downcase_word (html->engine, FALSE);
+		break;
+	default:
+		return;
+	}
+
+	html->binding_handled = TRUE;
+}
+
+/*
+  default keybindings:
+
+  Viewer:
+
+  Up/Down ............................. scroll one line up/down
+  PageUp/PageDown ..................... scroll one page up/down
+  Space/BackSpace ..................... scroll one page up/down
+
+  Left/Right .......................... scroll one char left/right
+  Shift Left/Right .................... scroll more left/right
+
+  Editor:
+
+  Up/Down ............................. move cursor one line up/down
+  PageUp/PageDown ..................... move cursor one page up/down
+
+  Return .............................. insert paragraph
+  Delete .............................. delete one char
+  BackSpace ........................... delete one char backwards
+
+*/
+
+static void
+clean_bindings_set (GtkBindingSet *binding_set)
+{
+	GtkBindingEntry *cur;
+	GList *mods, *vals, *cm, *cv;
+
+	if (!binding_set)
+		return;
+
+	mods = NULL;
+	vals = NULL;
+	cur = binding_set->entries;
+	while (cur) {
+		mods = g_list_prepend (mods, GUINT_TO_POINTER (cur->modifiers));
+		vals = g_list_prepend (vals, GUINT_TO_POINTER (cur->keyval));
+		cur = cur->set_next;
+	}
+	cm = mods; cv = vals;
+	while (cm) {
+		gtk_binding_entry_remove (binding_set, GPOINTER_TO_UINT (cv->data), GPOINTER_TO_UINT (cm->data));
+		cv = cv->next; cm = cm->next;
+	}
+	g_list_free (mods);
+	g_list_free (vals);
+}
+
+static void
+load_keybindings (GtkHTMLClass *klass)
+{
+	GtkBindingSet *binding_set;
+	gchar *base = NULL;
+	gchar *rcfile, *name;
+
+	/* FIXME add to gtk gtk_binding_set_clear & gtk_binding_set_remove_path */
+	clean_bindings_set (gtk_binding_set_by_class (klass));
+	clean_bindings_set (gtk_binding_set_find ("gtkhtml-bindings-emacs"));
+	clean_bindings_set (gtk_binding_set_find ("gtkhtml-bindings-ms"));
+	clean_bindings_set (gtk_binding_set_find ("gtkhtml-bindings-custom"));
+
+	/* ensure enums are defined */
+	gtk_html_cursor_skip_get_type ();
+	gtk_html_command_get_type ();
+
+	if (strcmp (klass->properties->keybindings_theme, "custom")) {
+		base = g_strconcat ("keybindingsrc.", klass->properties->keybindings_theme, NULL);
+		rcfile = g_concat_dir_and_file (PREFIX "/share/gtkhtml", base);
+		g_free (base);
+	} else {
+		rcfile = g_strconcat (gnome_util_user_home (), "/.gnome/gtkhtml-bindings-custom", NULL);
+	}
+
+	if (g_file_exists (rcfile)) {
+
+		/* g_warning ("Loading keybindings -- %s", rcfile); */
+		gtk_rc_parse (rcfile);
+
+		name = g_strconcat ("gtkhtml-bindings-", klass->properties->keybindings_theme, NULL);
+		binding_set = gtk_binding_set_find (name);
+		/* g_warning ("Looking for %s set", name); */
+		g_assert (binding_set);
+		gtk_binding_set_add_path (binding_set,
+					  GTK_PATH_CLASS,
+					  gtk_type_name (GTK_OBJECT_CLASS (klass)->type),
+					  GTK_PATH_PRIO_GTK);
+		g_free (name);
+	} else
+		g_warning (_("Couldn't find keybinding file -- %s"), rcfile);
+	g_free (rcfile);
+
+	binding_set = gtk_binding_set_by_class (klass);
+
+	/* layout scrolling */
+#define BSCROLL(m,key,orient,sc) \
+	gtk_binding_entry_add_signal (binding_set, GDK_ ## key, m, \
+				      "scroll", 3, \
+				      GTK_TYPE_ORIENTATION, GTK_ORIENTATION_ ## orient, \
+				      GTK_TYPE_ENUM, GTK_SCROLL_ ## sc, \
+				      GTK_TYPE_FLOAT, 0.0); \
+
+	BSCROLL (0, Up, VERTICAL, STEP_BACKWARD);
+	BSCROLL (0, KP_Up, VERTICAL, STEP_BACKWARD);
+	BSCROLL (0, Down, VERTICAL, STEP_FORWARD);
+	BSCROLL (0, KP_Down, VERTICAL, STEP_FORWARD);
+
+	BSCROLL (0, Left, HORIZONTAL, STEP_BACKWARD);
+	BSCROLL (0, KP_Left, HORIZONTAL, STEP_BACKWARD);
+	BSCROLL (0, Right, HORIZONTAL, STEP_FORWARD);
+	BSCROLL (0, KP_Right, HORIZONTAL, STEP_FORWARD);
+
+	BSCROLL (0, Page_Up, VERTICAL, PAGE_BACKWARD);
+	BSCROLL (0, KP_Page_Up, VERTICAL, PAGE_BACKWARD);
+	BSCROLL (0, Page_Down, VERTICAL, PAGE_FORWARD);
+	BSCROLL (0, KP_Page_Down, VERTICAL, PAGE_FORWARD);
+	BSCROLL (0, BackSpace, VERTICAL, PAGE_BACKWARD);
+	BSCROLL (0, space, VERTICAL, PAGE_FORWARD);
+
+	BSCROLL (GDK_SHIFT_MASK, Left, HORIZONTAL, PAGE_BACKWARD);
+	BSCROLL (GDK_SHIFT_MASK, KP_Left, HORIZONTAL, PAGE_BACKWARD);
+	BSCROLL (GDK_SHIFT_MASK, Right, HORIZONTAL, PAGE_FORWARD);
+	BSCROLL (GDK_SHIFT_MASK, KP_Right, HORIZONTAL, PAGE_FORWARD);
+
+	/* editing */
+
+#define BMOVE(m,key,dir,sk) \
+	gtk_binding_entry_add_signal (binding_set, GDK_ ## key, m, \
+				      "cursor_move", 2, \
+				      GTK_TYPE_DIRECTION_TYPE, GTK_DIR_ ## dir, \
+				      GTK_TYPE_HTML_CURSOR_SKIP, GTK_HTML_CURSOR_SKIP_ ## sk);
+
+	BMOVE (0, Left,     LEFT,  ONE);
+	BMOVE (0, KP_Left,  LEFT,  ONE);
+	BMOVE (0, Right,    RIGHT, ONE);
+	BMOVE (0, KP_Right, RIGHT, ONE);
+	BMOVE (0, Up,       UP  ,  ONE);
+	BMOVE (0, KP_Up,    UP  ,  ONE);
+	BMOVE (0, Down,     DOWN,  ONE);
+	BMOVE (0, KP_Down,  DOWN,  ONE);
+
+	BMOVE (GDK_CONTROL_MASK, Left,  LEFT,  WORD);
+	BMOVE (GDK_CONTROL_MASK, KP_Left,  LEFT,  WORD);
+	BMOVE (GDK_CONTROL_MASK, h, RIGHT, WORD);
+	BMOVE (GDK_CONTROL_MASK, KP_Right, RIGHT, WORD);
+
+	BMOVE (0, Page_Up,       UP,   PAGE);
+	BMOVE (0, KP_Page_Up,    UP,   PAGE);
+	BMOVE (0, Page_Down,     DOWN, PAGE);
+	BMOVE (0, KP_Page_Down,  DOWN, PAGE);
+
+	BMOVE (GDK_CONTROL_MASK, a, LEFT,  ALL);
+	BMOVE (GDK_CONTROL_MASK, e, RIGHT, ALL);
+
+#define BCOM(m,key,com) \
+	gtk_binding_entry_add_signal (binding_set, GDK_ ## key, m, \
+				      "command", 1, \
+				      GTK_TYPE_HTML_COMMAND, GTK_HTML_COMMAND_ ## com);
+
+	BCOM (0, Return, INSERT_PARAGRAPH);
+	BCOM (0, BackSpace, DELETE_BACK);
+	BCOM (GDK_SHIFT_MASK, BackSpace, DELETE_BACK);
+	BCOM (0, Delete, DELETE);
+	BCOM (0, KP_Delete, DELETE);
 }
