@@ -37,6 +37,8 @@ typedef struct _GtkHTMLClass	GtkHTMLClass;
 #define GTK_IS_HTML(obj)               (GTK_CHECK_TYPE ((obj), GTK_TYPE_HTML))
 #define GTK_IS_HTML_CLASS(klass)       (GTK_CHECK_CLASS_TYPE ((klass), GTK_TYPE_HTML))
 
+#define GTK_HTML_PROPERTY(w,p)         (GTK_HTML_CLASS (GTK_OBJECT (w)->klass)->properties-> ## p)
+
 
 typedef struct _GtkHTMLStream GtkHTMLStream;
 
@@ -69,7 +71,92 @@ enum _GtkHTMLParagraphAlignment {
 };
 typedef enum _GtkHTMLParagraphAlignment GtkHTMLParagraphAlignment;
 
+enum _GtkHTMLCursorSkipType {
+	GTK_HTML_CURSOR_SKIP_ONE,
+	GTK_HTML_CURSOR_SKIP_WORD,
+	GTK_HTML_CURSOR_SKIP_PAGE,
+	GTK_HTML_CURSOR_SKIP_ALL,
+};
+typedef enum _GtkHTMLCursorSkipType GtkHTMLCursorSkipType;
+
+enum _GtkHTMLCommandType {
+	GTK_HTML_COMMAND_UNDO,
+	GTK_HTML_COMMAND_REDO,
+	GTK_HTML_COMMAND_COPY,
+	GTK_HTML_COMMAND_CUT,
+	GTK_HTML_COMMAND_PASTE,
+
+	GTK_HTML_COMMAND_CUT_LINE,
+
+	GTK_HTML_COMMAND_INSERT_PARAGRAPH,
+	GTK_HTML_COMMAND_INSERT_RULE,
+	GTK_HTML_COMMAND_INSERT_RULE_PARAM,
+	GTK_HTML_COMMAND_INSERT_IMAGE_PARAM,
+
+	GTK_HTML_COMMAND_MAKE_LINK,
+	GTK_HTML_COMMAND_REMOVE_LINK,
+
+	GTK_HTML_COMMAND_DELETE,
+	GTK_HTML_COMMAND_DELETE_BACK,
+
+	GTK_HTML_COMMAND_SET_MARK,
+	GTK_HTML_COMMAND_DISABLE_SELECTION,
+
+	GTK_HTML_COMMAND_TOGGLE_BOLD,
+	GTK_HTML_COMMAND_TOGGLE_ITALIC,
+	GTK_HTML_COMMAND_TOGGLE_UNDERLINE,
+	GTK_HTML_COMMAND_TOGGLE_STRIKEOUT,
+
+	GTK_HTML_COMMAND_SIZE_MINUS_2,
+	GTK_HTML_COMMAND_SIZE_MINUS_1,
+	GTK_HTML_COMMAND_SIZE_PLUS_0,
+	GTK_HTML_COMMAND_SIZE_PLUS_1,
+	GTK_HTML_COMMAND_SIZE_PLUS_2,
+	GTK_HTML_COMMAND_SIZE_PLUS_3,
+	GTK_HTML_COMMAND_SIZE_PLUS_4,
+
+	GTK_HTML_COMMAND_SIZE_INCREASE,
+	GTK_HTML_COMMAND_SIZE_DECREASE,
+
+	GTK_HTML_COMMAND_ALIGN_LEFT,
+	GTK_HTML_COMMAND_ALIGN_CENTER,
+	GTK_HTML_COMMAND_ALIGN_RIGHT,
+
+	GTK_HTML_COMMAND_INDENT_INC,
+	GTK_HTML_COMMAND_INDENT_DEC,
+
+	GTK_HTML_COMMAND_PARAGRAPH_STYLE_NORMAL,
+	GTK_HTML_COMMAND_PARAGRAPH_STYLE_H1,
+	GTK_HTML_COMMAND_PARAGRAPH_STYLE_H2,
+	GTK_HTML_COMMAND_PARAGRAPH_STYLE_H3,
+	GTK_HTML_COMMAND_PARAGRAPH_STYLE_H4,
+	GTK_HTML_COMMAND_PARAGRAPH_STYLE_H5,
+	GTK_HTML_COMMAND_PARAGRAPH_STYLE_H6,
+	GTK_HTML_COMMAND_PARAGRAPH_STYLE_ADDRESS,
+	GTK_HTML_COMMAND_PARAGRAPH_STYLE_PRE,
+	GTK_HTML_COMMAND_PARAGRAPH_STYLE_ITEMDOTTED,
+	GTK_HTML_COMMAND_PARAGRAPH_STYLE_ITEMROMAN,
+	GTK_HTML_COMMAND_PARAGRAPH_STYLE_ITEMDIGIT,
+
+	GTK_HTML_COMMAND_MODIFY_SELECTION_UP,
+	GTK_HTML_COMMAND_MODIFY_SELECTION_DOWN,
+	GTK_HTML_COMMAND_MODIFY_SELECTION_LEFT,
+	GTK_HTML_COMMAND_MODIFY_SELECTION_RIGHT,
+	GTK_HTML_COMMAND_MODIFY_SELECTION_PAGEUP,
+	GTK_HTML_COMMAND_MODIFY_SELECTION_PAGEDOWN,
+	GTK_HTML_COMMAND_MODIFY_SELECTION_BOL,
+	GTK_HTML_COMMAND_MODIFY_SELECTION_EOL,
+	GTK_HTML_COMMAND_MODIFY_SELECTION_BOD,
+	GTK_HTML_COMMAND_MODIFY_SELECTION_EOD,
+
+	GTK_HTML_COMMAND_CAPITALIZE_WORD,
+	GTK_HTML_COMMAND_UPCASE_WORD,
+	GTK_HTML_COMMAND_DOWNCASE_WORD,
+};
+typedef enum _GtkHTMLCommandType GtkHTMLCommandType;
+
 
+#include "gtkhtml-properties.h"
 #include "gtkhtml-embedded.h"
 
 /* FIXME we should hide this stuff.  */
@@ -113,6 +200,8 @@ struct _GtkHTML {
 	GtkHTMLParagraphAlignment paragraph_alignment;
 
 	GtkHTMLFontStyle insertion_font_style;
+
+	gboolean binding_handled;
 };
 
 /* must be forward referenced *sigh* */
@@ -137,9 +226,21 @@ struct _GtkHTMLClass {
 	void (* current_paragraph_alignment_changed) (GtkHTML *html, GtkHTMLParagraphAlignment new_alignment);
 	void (* current_paragraph_indentation_changed) (GtkHTML *html, guint new_indentation);
 	void (* insertion_font_style_changed) (GtkHTML *html, GtkHTMLFontStyle style);
+	void (* insertion_color_changed) (GtkHTML *html, GdkColor *color);
+
+        void (* size_changed)       (GtkHTML *html);
+
+	/* keybindings signals */
+	void (* scroll)               (GtkHTML *html, GtkOrientation orientation, GtkScrollType scroll_type, gfloat position);
+	void (* cursor_move)          (GtkHTML *html, GtkDirectionType dir_type, GtkHTMLCursorSkipType skip);
+	void (* command)              (GtkHTML *html, GtkHTMLCommandType com_type);
+
+	/* properties */
+	GtkHTMLClassProperties *properties;
 };
 
 
+
 /* Creation.  */
 GtkType    gtk_html_get_type  (void);
 GtkWidget *gtk_html_new       (void);
@@ -199,15 +300,19 @@ gboolean  gtk_html_jump_to_anchor  (GtkHTML *html,
 
 /* Editing functions.  */
 
-void  gtk_html_set_paragraph_style  (GtkHTML                   *html,
-				     GtkHTMLParagraphStyle      style);
-void  gtk_html_indent               (GtkHTML                   *html,
-				     gint                       delta);
-void  gtk_html_set_font_style       (GtkHTML                   *html,
-				     GtkHTMLFontStyle           and_mask,
-				     GtkHTMLFontStyle           or_mask);
-void  gtk_html_align_paragraph      (GtkHTML                   *html,
-				     GtkHTMLParagraphAlignment  alignment);
+GtkHTMLParagraphStyle	   gtk_html_get_paragraph_style          (GtkHTML                   *html);
+void  			   gtk_html_set_paragraph_style          (GtkHTML                   *html,
+								  GtkHTMLParagraphStyle      style);
+void  			   gtk_html_indent                       (GtkHTML                   *html,
+								  gint                       delta);
+void  			   gtk_html_set_font_style               (GtkHTML                   *html,
+								  GtkHTMLFontStyle           and_mask,
+								  GtkHTMLFontStyle           or_mask);
+void                       gtk_html_toggle_font_style            (GtkHTML                   *html,
+								  GtkHTMLFontStyle           style);
+GtkHTMLParagraphAlignment  gtk_html_get_paragraph_alignment      (GtkHTML                   *html);
+void  			   gtk_html_set_paragraph_alignment      (GtkHTML                   *html,
+								  GtkHTMLParagraphAlignment  alignment);
 
 void  gtk_html_cut    (GtkHTML *html);
 void  gtk_html_copy   (GtkHTML *html);
