@@ -118,76 +118,96 @@ is_header (HTMLClueFlow *flow)
 	}
 }
 
-static void
-add_pre_padding (HTMLClueFlow *flow,
+static guint
+get_pre_padding (HTMLClueFlow *flow,
 		 guint pad)
 {
 	HTMLObject *prev_object;
 
 	prev_object = HTML_OBJECT (flow)->prev;
 	if (prev_object == NULL)
-		return;
+		return 0;
 
 	if (HTML_OBJECT_TYPE (prev_object) == HTML_TYPE_CLUEFLOW) {
 		HTMLClueFlow *prev;
 
 		prev = HTML_CLUEFLOW (prev_object);
 		if (prev->level > 0 && flow->level == 0)
-			goto add_pad;
+			return pad;
 
 		if (flow->style == HTML_CLUEFLOW_STYLE_PRE
 		    && prev->style != HTML_CLUEFLOW_STYLE_PRE
 		    && ! is_header (prev))
-			goto add_pad;
+			return pad;
 
 		if (is_header (flow) && ! is_header (prev))
-			goto add_pad;
+			return pad;
 
-		return;
+		return 0;
 	}
 
 	if (! is_header (flow) && flow->level == 0)
-		return;
+		return 0;
 
- add_pad:
-	HTML_OBJECT (flow)->ascent += pad;
-	HTML_OBJECT (flow)->y += pad;
+	return pad;
 }
 
-static void
-add_post_padding (HTMLClueFlow *flow,
+static guint
+get_post_padding (HTMLClueFlow *flow,
 		  guint pad)
 {
 	HTMLObject *next_object;
 
 	next_object = HTML_OBJECT (flow)->next;
 	if (next_object == NULL)
-		return;
+		return 0;
 
 	if (HTML_OBJECT_TYPE (next_object) == HTML_TYPE_CLUEFLOW) {
 		HTMLClueFlow *next;
 
 		next = HTML_CLUEFLOW (next_object);
 		if (next->level > 0 && flow->level == 0)
-			goto add_pad;
+			return pad;
 
 		if (flow->style == HTML_CLUEFLOW_STYLE_PRE
 		    && next->style != HTML_CLUEFLOW_STYLE_PRE
 		    && ! is_header (next))
-			goto add_pad;
+			return pad;
 
 		if (is_header (flow) &&  ! is_header (next))
-			goto add_pad;
+			return pad;
 
-		return;
+		return 0;
 	}
 
 	if (! is_header (flow) && flow->level == 0)
-		return;
+		return 0;
 
- add_pad:
-	HTML_OBJECT (flow)->ascent += pad;
-	HTML_OBJECT (flow)->y += pad;
+	return pad;
+}
+
+static void
+add_pre_padding (HTMLClueFlow *flow,
+		 guint pad)
+{
+	guint real_pad;
+
+	real_pad = get_pre_padding (flow, pad);
+
+	HTML_OBJECT (flow)->ascent += real_pad;
+	HTML_OBJECT (flow)->y += real_pad;
+}
+
+static void
+add_post_padding (HTMLClueFlow *flow,
+		  guint pad)
+{
+	guint real_pad;
+
+	real_pad = get_post_padding (flow, pad);
+
+	HTML_OBJECT (flow)->ascent += real_pad;
+	HTML_OBJECT (flow)->y += real_pad;
 }
 
 static guint
@@ -220,9 +240,10 @@ set_max_width (HTMLObject *o,
 	o->max_width = max_width;
 
 	indent = get_indent (HTML_CLUEFLOW (o), painter);
-
-	for (obj = HTML_CLUE (o)->head; obj != 0; obj = obj->next)
+	
+	for (obj = HTML_CLUE (o)->head; obj != 0; obj = obj->next) {
 		html_object_set_max_width (obj, painter, o->max_width - indent);
+	}
 }
 
 
@@ -352,6 +373,8 @@ calc_size (HTMLObject *o,
 	gint runWidth = 0;
 	gboolean have_valign_top;
 
+	html_clueflow_remove_text_slaves (HTML_CLUEFLOW (o));
+
 	changed = FALSE;
 	old_ascent = o->ascent;
 	old_descent = o->descent;
@@ -802,7 +825,8 @@ draw (HTMLObject *self,
 
 		xp += tx, yp += ty;
 
-		html_painter_set_pen (painter, html_painter_get_black (painter));
+		html_painter_set_pen (painter,
+				      &html_colorset_get_color_allocated (painter, HTMLTextColor)->color);
 
 		if (clueflow->level == 0 || (clueflow->level & 1) != 0)
 			html_painter_fill_rect (painter, xp, yp, bullet_size, bullet_size);
@@ -817,6 +841,30 @@ draw (HTMLObject *self,
 							tx, ty);
 }
 
+static void
+draw_background (HTMLObject *self,
+		 HTMLPainter *p,
+		 gint x, gint y,
+		 gint width, gint height,
+		 gint tx, gint ty)
+{
+	
+	html_object_draw (self->parent, p,
+			  x + self->parent->x,
+			  y + self->parent->y - self->parent->ascent,
+			  width, height,
+			  tx - self->parent->x,
+			  ty - self->parent->y + self->parent->ascent);
+	
+	html_object_draw_background (self->parent, p,
+				     x + self->parent->x,
+				     y + self->parent->y - self->parent->ascent,
+				     width, height,
+				     tx - self->parent->x,
+				     ty - self->parent->y + self->parent->ascent);
+}
+
+
 static HTMLObject*
 check_point (HTMLObject *self,
 	     HTMLPainter *painter,
@@ -830,7 +878,7 @@ check_point (HTMLObject *self,
 	HTMLClue *clue;
 
 	if (x < self->x || x >= self->x + self->width
-	    || y < self->y - self->ascent + 1 || y > self->y + self->descent)
+	    || y < self->y - self->ascent || y >= self->y + self->descent)
 		return NULL;
 
 	clue = HTML_CLUE (self);
@@ -889,7 +937,6 @@ append_selection_string (HTMLObject *self,
 	if (! self->selected)
 		return;
 
-	putchar ('\n');
 	g_string_append_c (buffer, '\n');
 }
 
@@ -897,17 +944,53 @@ append_selection_string (HTMLObject *self,
 /* Saving support.  */
 
 static gboolean
-write_indentation_tags_helper (HTMLEngineSaveState *state,
-			       guint last_value,
-			       guint new_value,
-			       const gchar *tag)
+write_indent (HTMLEngineSaveState *state)
+{
+	return html_engine_save_output_string (state, "    ");
+}
+
+static const char *
+get_tag (HTMLClueFlow *flow)
+{
+	switch (flow->style) {
+	case HTML_CLUEFLOW_STYLE_ITEMDOTTED:
+		return "UL";
+	case HTML_CLUEFLOW_STYLE_ITEMROMAN:
+	case HTML_CLUEFLOW_STYLE_ITEMDIGIT:
+		return "OL";
+	case HTML_CLUEFLOW_STYLE_NORMAL:
+	case HTML_CLUEFLOW_STYLE_H1:
+	case HTML_CLUEFLOW_STYLE_H2:
+	case HTML_CLUEFLOW_STYLE_H3:
+	case HTML_CLUEFLOW_STYLE_H4:
+	case HTML_CLUEFLOW_STYLE_H5:
+	case HTML_CLUEFLOW_STYLE_H6:
+	case HTML_CLUEFLOW_STYLE_ADDRESS:
+	case HTML_CLUEFLOW_STYLE_PRE:
+	case HTML_CLUEFLOW_STYLE_NOWRAP:
+	default:
+		return "BLOCKQUOTE";
+	}
+}
+
+static gboolean
+write_indentation_tags (HTMLEngineSaveState *state,
+			guint last_value,
+			guint new_value,
+			const gchar *tag)
 {
 	guint i, j;
 
+	if (new_value == last_value)
+		return TRUE;
+
+	if (! html_engine_save_output_string (state, "\n"))
+		return FALSE;
+
 	if (new_value > last_value) {
 		for (i = last_value; i < new_value; i++) {
 			for (j = 0; j < i; j++) {
-				if (! html_engine_save_output_string (state, "    "))
+				if (! write_indent (state))
 					return FALSE;
 			}
 
@@ -932,21 +1015,49 @@ write_indentation_tags_helper (HTMLEngineSaveState *state,
 }
 
 static gboolean
-write_indentation_tags (HTMLClueFlow *self,
-			HTMLEngineSaveState *state)
+write_pre_tags (HTMLClueFlow *self,
+		HTMLEngineSaveState *state)
 {
-	if (state->last_level == self->level)
+	HTMLClueFlow *prev;
+	const char *prev_tag, *curr_tag;
+
+	prev = HTML_CLUEFLOW (HTML_OBJECT (self)->prev);
+	if (prev != NULL && prev->level == self->level && prev->style == self->style) {
+		if (! is_item (self))
+			return html_engine_save_output_string (state, "<BR>\n");
+		else
+			return TRUE;
+	}
+
+	if (prev != NULL)
+		prev_tag = get_tag (prev);
+	else
+		prev_tag = NULL;
+
+	curr_tag = get_tag (self);
+
+	if (prev != NULL && strcmp (prev_tag, curr_tag) == 0) {
+		write_indentation_tags (state, prev->level, self->level, prev_tag);
+	} else {
+		if (prev != NULL)
+			write_indentation_tags (state, prev->level, 0, prev_tag);
+		write_indentation_tags (state, 0, self->level, curr_tag);
+	}
+
+	return TRUE;
+}
+
+static gboolean
+write_post_tags (HTMLClueFlow *self,
+		 HTMLEngineSaveState *state)
+{
+	const char *tag;
+
+	if (HTML_OBJECT (self)->next != NULL)
 		return TRUE;
 
-	/* FIXME should use UL appropriately.  */
-
-	if (! write_indentation_tags_helper (state,
-					     state->last_level,
-					     self->level,
-					     "BLOCKQUOTE"))
-		return FALSE;
-
-	state->last_level = self->level;
+	tag = get_tag (self);
+	write_indentation_tags (state, self->level, 0, tag);
 
 	return TRUE;
 }
@@ -983,21 +1094,6 @@ get_tag_for_style (const HTMLClueFlow *flow)
 	}
 }
 
-static const gchar *
-get_tag_for_item_group (const HTMLClueFlow *flow)
-{
-	switch (flow->style) {
-	case HTML_CLUEFLOW_STYLE_ITEMDOTTED:
-		return "UL";
-	case HTML_CLUEFLOW_STYLE_ITEMROMAN:
-	case HTML_CLUEFLOW_STYLE_ITEMDIGIT:
-		return "OL";
-	default:
-		g_warning ("Unknown HTMLClueFlowStyle %d", flow->style);
-		return NULL;
-	}
-}
-
 static const gchar *
 halign_to_string (HTMLHAlignType halign)
 {
@@ -1019,7 +1115,7 @@ is_similar (HTMLObject *self, HTMLObject *friend)
 	if (friend &&  HTML_OBJECT_TYPE (friend) == HTML_TYPE_CLUEFLOW) {
 		if ((HTML_CLUEFLOW (friend)->style == HTML_CLUEFLOW (self)->style)
 		    && (HTML_CLUEFLOW (friend)->level == HTML_CLUEFLOW (self)->level)) {
-				return TRUE;
+			return TRUE;
 		}
 	}
 	return FALSE;
@@ -1033,11 +1129,12 @@ save (HTMLObject *self,
 	HTMLHAlignType halign;
 	const gchar *tag;
 	gboolean start = TRUE, end = TRUE;
+	gint i;
 
 	clueflow = HTML_CLUEFLOW (self);
 	halign = HTML_CLUE (self)->halign;
 
-	if (! write_indentation_tags (clueflow, state))
+	if (! write_pre_tags (clueflow, state))
 		return FALSE;
 
 	tag = get_tag_for_style (clueflow);
@@ -1047,33 +1144,21 @@ save (HTMLObject *self,
 
 	if (is_similar (self, self->next))
 		end = FALSE;
-	
-	if (self->prev != NULL && ! is_item (clueflow)
-	    && HTML_OBJECT_TYPE (HTML_CLUE (self)->tail) != HTML_TYPE_RULE) {
-		if (HTML_CLUEFLOW (self)->style != HTML_CLUEFLOW_STYLE_PRE) {
-			/* This is a nasty hack: the rule takes all of the space, so we
-			   don't want it to create a new newline if it's the last
-			   element on the paragraph.  Also, everything but the "normal"
-			   paragraph style would get an extra newline if we add a
-			   `<BR>'.  */
-			html_engine_save_output_string (state, "<BR>\n");
-		} else {
-			html_engine_save_output_string (state, "\n");
+
+	/* Indentation.  */
+	if (clueflow->style != HTML_CLUEFLOW_STYLE_PRE) {
+		for (i = 0; i < clueflow->level; i++) {
+			if (! write_indent (state))
+				return FALSE;
 		}
 	}
-
+	
 	/* Alignment tag.  */
 	if (halign != HTML_HALIGN_NONE && halign != HTML_HALIGN_LEFT) {
 		if (! html_engine_save_output_string (state, "<DIV ALIGN=%s>\n", halign_to_string (halign)))
 			return FALSE;
 	}
 
-	if (start && is_item (clueflow)) {
-		if (! html_engine_save_output_string (state, "<%s>\n",
-						      get_tag_for_item_group (HTML_CLUEFLOW (self))))
-			return FALSE;
-	}		
-
 	/* Start tag.  */
 	if (tag != NULL && (start || is_item (clueflow))
 	    && (! html_engine_save_output_string (state, "<%s>", tag)))
@@ -1089,28 +1174,39 @@ save (HTMLObject *self,
 			return FALSE;
 	}
 
-	if (end && is_item (clueflow)) {
-		if (! html_engine_save_output_string (state, "\n</%s>",
-						      get_tag_for_item_group (HTML_CLUEFLOW (self))))
-			return FALSE;
-	}		
-
 	/* Close alignment tag.  */
 	if (halign != HTML_HALIGN_NONE && halign != HTML_HALIGN_LEFT) {
 		if (! html_engine_save_output_string (state, "</DIV>\n"))
 			return FALSE;
-	} else if (tag != NULL) {
+	} else if (tag != NULL
+		   && HTML_CLUEFLOW (self)->style != HTML_CLUEFLOW_STYLE_PRE) {
 		if (! html_engine_save_output_string (state, "\n"))
 			return FALSE;
 	}
 
-	return TRUE;
+	return write_post_tags (HTML_CLUEFLOW (self), state);
 }
 
 static gboolean
 save_plain (HTMLObject *self,
 	    HTMLEngineSaveState *state)
 {
+	HTMLClueFlow *clueflow;
+
+	clueflow = HTML_CLUEFLOW (self);
+
+	if (get_pre_padding (clueflow, 1) > 0)
+		if (! html_engine_save_output_string (state, "\n"))
+			return FALSE;
+
+	if (clueflow->level > 0) {
+		gint i;
+
+		for (i = 0; i < (gint) clueflow->level; i++) {
+			if (! html_engine_save_output_string (state, "\t"))
+				return FALSE;
+		}
+	}
 
 	/* Paragraph's content.  */
 	if (! HTML_OBJECT_CLASS (&html_clue_class)->save_plain (self, state))
@@ -1119,6 +1215,10 @@ save_plain (HTMLObject *self,
 	if (!html_engine_save_output_string (state, "\n"))
 		return FALSE;
 	
+	if (get_post_padding (clueflow, 1) > 0)
+		if (! html_engine_save_output_string (state, "\n"))
+			return FALSE;
+
 	return TRUE;
 }
 
@@ -1191,8 +1291,6 @@ search_set_info (HTMLObject *cur, HTMLSearch *info, guint pos, guint len)
 	guint text_len = 0;
 	guint cur_len;
 
-	printf ("se_info pos: %d len: %d\n", pos, len);
-
 	info->found_len = len;
 
 	if (info->found) {
@@ -1278,8 +1376,6 @@ search_text (HTMLObject **beg, HTMLSearch *info)
 			cur = (info->forward) ? cur->next : cur->prev;
 		}
 
-		printf ("text (%d): %s\n", text_len, par);
-
 		/* set eq_len and pos counters */
 		eq_len = 0;
 		if (info->found) {
@@ -1311,13 +1407,10 @@ search_text (HTMLObject **beg, HTMLSearch *info)
 
 				while ((info->forward && pos < text_len)
 				       || (!info->forward && pos >= 0)) {
-					printf ("try match pos: %d %s\n", pos, par + pos);
 					rv = regexec (info->reb,
 						      par + pos,
 						      1, &match, 0);
 					if (rv == 0) {
-						printf ("found! pos: %d start: %d len: %d\n",
-							pos, pos + match.rm_so, match.rm_eo - match.rm_so);
 						search_set_info (head, info, pos + match.rm_so, match.rm_eo - match.rm_so);
 						retval = TRUE;
 						break;
@@ -1327,14 +1420,12 @@ search_text (HTMLObject **beg, HTMLSearch *info)
 #else
 				rv = re_search (info->reb, par, text_len, pos,
 						(info->forward) ? text_len-pos : -pos, NULL);
-				printf ("search result: %d pos: %d\n", rv, pos);
 				if (rv>=0) {
 					guint found_pos = rv;
 					rv = re_match (info->reb, par, text_len, found_pos, NULL);
 					if (rv < 0) {
 						g_warning ("re_match (...) error");
 					}
-					printf ("found! start: %d len: %d\n", found_pos, rv);
 					search_set_info (head, info, found_pos, rv);
 					retval = TRUE;
 				} else {
@@ -1352,9 +1443,8 @@ search_text (HTMLObject **beg, HTMLSearch *info)
 					    == info->trans [par [pos]]) {
 						eq_len++;
 						if (eq_len == info->text_len) {
-							printf ("found! pos %d\n", pos);
 							search_set_info (head, info, pos - ((info->forward)
-									 ? eq_len-1 : 0), info->text_len);
+											    ? eq_len-1 : 0), info->text_len);
 							retval=TRUE;
 							break;
 						}
@@ -1381,13 +1471,10 @@ search (HTMLObject *obj, HTMLSearch *info)
 	HTMLObject *cur;
 	gboolean next = FALSE;
 
-	printf ("search clueflow\n");
-
 	/* does last search end here? */
 	if (info->found) {
 		cur  = HTML_OBJECT (info->found->data);
 		next = TRUE;
-		printf ("search next clueflow continue\n");
 	} else {
 		/* search_next? */
 		if (html_search_child_on_stack (info, obj)) {
@@ -1461,6 +1548,7 @@ html_clueflow_class_init (HTMLClueFlowClass *klass,
 	object_class->calc_min_width = calc_min_width;
 	object_class->calc_preferred_width = calc_preferred_width;
 	object_class->draw = draw;
+	object_class->draw_background = draw_background;
 	object_class->save = save;
 	object_class->save_plain = save_plain;
 	object_class->check_page_split = check_page_split;
@@ -1587,8 +1675,8 @@ relayout_and_draw (HTMLObject *object,
    paragraph and the following one, because their padding might change
    after the level change. */
 static void
-relayout_for_level_change (HTMLClueFlow *flow,
-			   HTMLEngine *engine)
+relayout_with_siblings (HTMLClueFlow *flow,
+			HTMLEngine *engine)
 {
 	if (engine == NULL)
 		return;
@@ -1616,7 +1704,7 @@ html_clueflow_set_style (HTMLClueFlow *flow,
 
 	flow->style = style;
 
-	relayout_and_draw (HTML_OBJECT (flow), engine);
+	relayout_with_siblings (flow, engine);
 }
 
 HTMLClueFlowStyle
@@ -1672,7 +1760,7 @@ html_clueflow_indent (HTMLClueFlow *flow,
 		return;
 	}
 
-	relayout_for_level_change (flow, engine);
+	relayout_with_siblings (flow, engine);
 }
 
 void
@@ -1689,7 +1777,7 @@ html_clueflow_set_indentation (HTMLClueFlow *flow,
 
 	flow->level = indentation;
 
-	relayout_for_level_change (flow, engine);
+	relayout_with_siblings (flow, engine);
 }
 
 guint8
