@@ -51,6 +51,7 @@ safe_remove (HTMLEngine *e,
 {
 	HTMLObject *p;
 	HTMLCursor *cursor;
+	HTMLObject *next;
 
 	cursor = e->cursor;
 
@@ -72,12 +73,10 @@ safe_remove (HTMLEngine *e,
 				e->cursor->offset = 0;
 			} else {
 				HTMLObject *master;
-				GdkColor black = { 0, 0, 0 };
 
-				/* FIXME black.  */
 				master = html_text_master_new ("",
 							       GTK_HTML_FONT_STYLE_DEFAULT,
-							       &black);
+							       e->insertion_color);
 				html_clue_prepend (HTML_CLUE (object->parent), master);
 
 				e->cursor->object = master;
@@ -88,7 +87,8 @@ safe_remove (HTMLEngine *e,
 
 	for (p = object->next;
 	     p != NULL && HTML_OBJECT_TYPE (p) == HTML_TYPE_TEXTSLAVE;
-	     p = p->next) {
+	     p = next) {
+		next = p->next;
 		html_clue_remove (HTML_CLUE (p->parent), p);
 		html_object_destroy (p);
 	}
@@ -229,10 +229,10 @@ append_to_buffer (GList **buffer,
 		return;
 
 	/* debug msg */
-	g_print ("Adding object %p [%s] to cut_buffer.\n",
+	/* g_print ("Adding object %p [%s] to cut_buffer.\n",
 		 object, html_type_name (HTML_OBJECT_TYPE (object)));
 	if (html_object_is_text (object))
-		g_print ("\ttext `%s'\n", HTML_TEXT (object)->text);
+	g_print ("\ttext `%s'\n", HTML_TEXT (object)->text); */
 
 	if (*buffer == NULL) {
 		*buffer = *buffer_tail = g_list_append (NULL, object);
@@ -333,9 +333,8 @@ delete_different_parent (HTMLEngine *e,
 	/* First destroy the elements in the `start_object's paragraph.  */
 
 	while (p != NULL) {
-		html_clue_remove (HTML_CLUE (start_object->parent), p);
-
 		pnext = p->next;
+		html_clue_remove (HTML_CLUE (start_object->parent), p);
 
 		if (HTML_OBJECT_TYPE (p) == HTML_TYPE_TEXTSLAVE)
 			html_object_destroy (p);
@@ -563,6 +562,8 @@ html_engine_delete (HTMLEngine *e,
 	g_return_if_fail (e != NULL);
 	g_return_if_fail (HTML_IS_ENGINE (e));
 
+	/* g_print ("Deleting %d %s\n", count, backwards ? "backwards" : "forwards"); */
+
 	if (e->cursor->object->parent == NULL || e->cursor->object->parent == NULL)
 		return;
 
@@ -629,6 +630,10 @@ html_engine_delete (HTMLEngine *e,
 	html_object_relayout (e->cursor->object->parent->parent, e,
 			      e->cursor->object->parent);
 
+	if (e->cursor->object->parent->next != NULL)
+		html_object_relayout (e->cursor->object->parent->next->parent, e,
+				      e->cursor->object->parent->next);
+
 	e->cursor->position = start_position;
 
 	html_cursor_normalize (e->cursor);
@@ -656,3 +661,41 @@ html_engine_delete (HTMLEngine *e,
 }
 
 
+/**
+ * html_engine_delete_selection:
+ * @e: An HTMLEngine
+ * @do_undo: Whether to save undo information for this operation
+ * 
+ * Delete the current selection (if any) in @e.  If there is no selection, this
+ * is a no-op.
+ **/
+void
+html_engine_delete_selection (HTMLEngine *e,
+			      gboolean do_undo)
+{
+	gboolean backwards;
+	guint count;
+
+	g_return_if_fail (e != NULL);
+	g_return_if_fail (HTML_IS_ENGINE (e));
+
+	html_engine_edit_selection_updater_update_now (e->selection_updater);
+
+	if (e->mark == NULL || e->mark->position == e->cursor->position)
+		return;
+
+	if (e->mark->position > e->cursor->position) {
+		count = e->mark->position - e->cursor->position;
+		backwards = FALSE;
+	} else {
+		count = e->cursor->position - e->mark->position;
+		backwards = TRUE;
+	}
+
+	if (count == 0)
+		return;
+
+	html_engine_disable_selection (e);
+
+	html_engine_delete (e, count, do_undo, backwards);
+}
