@@ -33,7 +33,7 @@ void  html_engine_insert_image  (HTMLEngine     *e,
 				 gint16          height,
 				 gint8           percent,
 				 gint8           border,
-				 const GdkColor *border_color,
+				 HTMLColor *border_color,
 				 HTMLHAlignType  halign,
 				 HTMLVAlignType  valign,
 				 gint8           hspace,
