@@ -25,6 +25,7 @@
 #include "htmltextmaster.h"
 
 #include "htmlengine-cutbuffer.h"
+#include "htmlengine-edit.h"
 #include "htmlengine-edit-clueflowstyle.h"
 #include "htmlengine-edit-cut.h"
 #include "htmlengine-edit-movement.h"
@@ -36,18 +37,19 @@
 
 /* #define PARANOID_DEBUG */
 
+static gboolean remove_element_if_empty_text (HTMLEngine *engine, HTMLObject *object);
+
 
 /* This function adds an empty HTMLTextMaster object to @flow; it is
    used to make sure no empty HTMLClueFlow object is created.  */
 static void
-add_empty_text_master_to_clueflow (HTMLClueFlow *flow)
+add_empty_text_master_to_clueflow (HTMLEngine *e, HTMLClueFlow *flow)
 {
-	static GdkColor black = { 0, 0, 0, 0 };	/* FIXME */
 	HTMLObject *new_textmaster;
 
 	new_textmaster = html_text_master_new ("",
 					       GTK_HTML_FONT_STYLE_DEFAULT,
-					       &black);
+					       html_colorset_get_color (e->settings->color_set, HTMLTextColor));
 	html_clue_prepend (HTML_CLUE (flow), new_textmaster);
 }
 
@@ -122,21 +124,20 @@ split_at_cursor (HTMLEngine *engine)
    with properties given by those of @clue.  */
 static void
 split_first_clueflow_at_cursor (HTMLEngine *engine,
-				HTMLClueFlow *clue)
+				HTMLClueFlow *clue,
+				HTMLObject *curr)
 {
 	HTMLObject *new_clueflow;
-	HTMLObject *curr;
 	HTMLObject *curr_clue;
 	HTMLObject *p, *pnext;
 
-	curr = engine->cursor->object;
 	curr_clue = curr->parent;
 
 	/* If there is nothing else before this element in the
            paragraph, add an empty text element, as HTMLClueFlows must
            never be empty.  */
 	if (curr->prev == NULL) {
-		add_empty_text_master_to_clueflow (HTML_CLUEFLOW (curr_clue));
+		add_empty_text_master_to_clueflow (engine, HTML_CLUEFLOW (curr_clue));
 		engine->cursor->object = HTML_CLUE (curr_clue)->head;
 		engine->cursor->offset = 0;
 	} else {
@@ -151,11 +152,8 @@ split_first_clueflow_at_cursor (HTMLEngine *engine,
 	new_clueflow = html_object_dup (HTML_OBJECT (clue));
 	html_object_change_set (new_clueflow, HTML_CHANGE_ALL);
 
-	if (curr_clue->prev == NULL)
-		html_clue_prepend (HTML_CLUE (curr_clue->parent), new_clueflow);
-	else
-		html_clue_append_after (HTML_CLUE (curr_clue->parent), new_clueflow,
-					curr_clue->prev);
+	html_clue_append_after (HTML_CLUE (curr_clue->parent), new_clueflow,
+				curr_clue->prev);
 
 	/* Move the stuff until the cursor position into the new HTMLClueFlow.  */
 
@@ -215,7 +213,7 @@ prepare_clueflows (HTMLEngine *engine,
 			continue;
 
 		if (first && ! append) {
-			split_first_clueflow_at_cursor (engine, HTML_CLUEFLOW (obj));
+			split_first_clueflow_at_cursor (engine, HTML_CLUEFLOW (obj), engine->cursor->object);
 
 			clue = engine->cursor->object->parent;
 			g_assert (clue != NULL);
@@ -224,12 +222,13 @@ prepare_clueflows (HTMLEngine *engine,
 			retval = TRUE;
 		} else {	/* ! first || append */
 			if (first) { /* first && append */
+				split_first_clueflow_at_cursor (engine, HTML_CLUEFLOW (obj), engine->cursor->object->next);
 				clue = engine->cursor->object->parent;
 				first = FALSE;
+			} else {
+				g_assert (clue != NULL);
+				clue = add_new_clueflow (engine, HTML_CLUEFLOW (obj), clue);
 			}
-
-			g_assert (clue != NULL);
-			clue = add_new_clueflow (engine, HTML_CLUEFLOW (obj), clue);
 		}
 	}
 
@@ -274,10 +273,7 @@ prepend_object (HTMLEngine *engine,
 	curr = cursor->object;
 	parent = curr->parent;
 
-	if (curr->prev == NULL)
-		html_clue_prepend (HTML_CLUE (parent), object);
-	else
-		html_clue_append_after (HTML_CLUE (parent), object, curr->prev);
+	html_clue_append_after (HTML_CLUE (parent), object, curr->prev);
 
 	cursor->position += obj_len (object);
 }
@@ -292,6 +288,9 @@ merge_possibly (HTMLEngine *engine,
 	if (a == NULL || b == NULL)
 		return;
 
+	if (remove_element_if_empty_text (engine, a) || remove_element_if_empty_text (engine, b))
+		return;
+
 	if (! html_object_is_text (a) || ! html_object_is_text (b))
 		return;
 
@@ -356,16 +355,16 @@ move_cursor_to_safe_object (HTMLEngine *engine,
 	g_assert_not_reached ();
 }
 
-static void
+static gboolean
 remove_element_if_empty_text (HTMLEngine *engine,
 			      HTMLObject *object)
 {
 	if (object == NULL)
-		return;
+		return FALSE;
 	if (! html_object_is_text (object))
-		return;
+		return FALSE;
 	if (HTML_TEXT (object)->text_len != 0)
-		return;
+		return FALSE;
 
 	if (engine->cursor->object == object) {
 		/* Of course, if we are destroying this object, it must be empty and
@@ -378,12 +377,14 @@ remove_element_if_empty_text (HTMLEngine *engine,
 	html_clue_remove (HTML_CLUE (object->parent), object);
 	html_text_master_destroy_slaves (HTML_TEXT_MASTER (object));
 	html_object_destroy (object);
+
+	return TRUE;
 }
 
 
 static guint
 do_paste (HTMLEngine *engine,
-	  GList *buffer)
+	      GList *buffer)
 {
 	/* Whether we need to append the elements at the cursor
 	   position or not.  */
@@ -425,9 +426,8 @@ do_paste (HTMLEngine *engine,
 #ifdef PARANOID_DEBUG
 	g_print ("\n**** Tree after clueflow preparation:\n\n");
 	gtk_html_debug_dump_tree_simple (engine->clue, 1);
-#endif
-
 	g_print ("\n");
+#endif
 
 	/* 5. Duplicate the objects in the cut buffer, one by one, and
 	   insert them into the document.  */
@@ -463,7 +463,7 @@ do_paste (HTMLEngine *engine,
 
 			while (p->next != NULL
 			       && HTML_OBJECT_TYPE (p->next->data) == HTML_TYPE_CLUEFLOW) {
-				add_empty_text_master_to_clueflow (HTML_CLUEFLOW (clueflow));
+				add_empty_text_master_to_clueflow (engine, HTML_CLUEFLOW (clueflow));
 
 				engine->cursor->position ++;
 
@@ -478,7 +478,7 @@ do_paste (HTMLEngine *engine,
 
 			if (p->next == NULL) {
 				if (HTML_CLUE (clueflow)->head == NULL)
-					add_empty_text_master_to_clueflow (HTML_CLUEFLOW (clueflow));
+					add_empty_text_master_to_clueflow (engine, HTML_CLUEFLOW (clueflow));
 				engine->cursor->object = HTML_CLUE (clueflow)->head;
 				engine->cursor->offset = 0;
 				break;
@@ -503,21 +503,17 @@ do_paste (HTMLEngine *engine,
 #endif
 		}
 
-		if (p->prev == NULL) {
-			if (html_object_is_text (obj_copy))
-				merge_possibly (engine, obj_copy->prev, obj_copy);
-			else
-				remove_element_if_empty_text
-					(engine, html_object_prev_not_slave (obj_copy));
-		}
+		if (html_object_is_text (obj_copy))
+			merge_possibly (engine, obj_copy->prev, obj_copy);
+		else
+			remove_element_if_empty_text
+				(engine, html_object_prev_not_slave (obj_copy));
 
-		if (p->next == NULL) {
-			if (html_object_is_text (obj_copy))
-				merge_possibly (engine, obj_copy, obj_copy->next);
-			else
-				remove_element_if_empty_text
-					(engine, html_object_next_not_slave (obj_copy));
-		}
+		if (html_object_is_text (obj_copy))
+			merge_possibly (engine, obj_copy, obj_copy->next);
+		else
+			remove_element_if_empty_text
+				(engine, html_object_next_not_slave (obj_copy));
 	}
 
 #ifdef PARANOID_DEBUG
@@ -532,7 +528,9 @@ do_paste (HTMLEngine *engine,
 	/* 9. Normalize the cursor pointer.  */
 	html_cursor_normalize (engine->cursor);
 
+#ifdef PARANOID_DEBUG
 	g_print ("%s -- %d element(s).\n", __FUNCTION__, engine->cursor->position - oldpos);
+#endif
 
 	/* Return the number of character elements pasted, to make undo/redo possible.  */
 	return engine->cursor->position - oldpos;
@@ -788,20 +786,35 @@ html_engine_paste_object (HTMLEngine *engine,
 			  HTMLObject *object,
 			  gboolean do_undo)
 {
-	GList *tmp_buffer;
-
 	g_return_if_fail (engine != NULL);
 	g_return_if_fail (HTML_IS_ENGINE (engine));
 	g_return_if_fail (object != NULL);
 
-	/* FIXME: Nasty, nasty, nasty hack.  */
+	html_engine_cut_buffer_push (engine);
 
-	tmp_buffer = engine->cut_buffer;
 	engine->cut_buffer = g_list_alloc ();
 	engine->cut_buffer->data = object;
 
 	html_engine_paste (engine, do_undo);
-
 	g_list_free (engine->cut_buffer);
-	engine->cut_buffer = tmp_buffer;
+
+	html_engine_cut_buffer_pop (engine);
+}
+
+void
+html_engine_replace_by_object (HTMLEngine *engine,
+			       HTMLObject *sobj, guint soff,
+			       HTMLObject *eobj, guint eoff,
+			       HTMLObject *new_obj)
+{
+	guint pos = engine->cursor->position;
+
+	html_engine_disable_selection (engine);
+	html_cursor_jump_to (engine->cursor, engine, sobj, soff);
+	html_engine_set_mark (engine);
+	html_cursor_jump_to (engine->cursor, engine, eobj, eoff);
+	html_engine_edit_selection_updater_update_now (engine->selection_updater);
+	html_engine_paste_object (engine, new_obj, TRUE);
+
+	html_cursor_jump_to_position (engine->cursor, engine, pos);
 }
