@@ -33,7 +33,6 @@ typedef struct _HTMLEngineClass HTMLEngineClass;
 #include "gtkhtml.h"
 
 #include "htmltokenizer.h"
-#include "htmlcolorset.h"
 #include "htmlcursor.h"
 #include "htmldrawqueue.h"
 #include "htmlstack.h"
@@ -75,8 +74,6 @@ typedef struct _HTMLBlockStackElement HTMLBlockStackElement;
 
 struct _HTMLEngine {
 	GtkObject parent;
-
-	HTMLColorSet *color_set;
 	HTMLDrawQueue *draw_queue;
 
 	HTMLPainter *painter;
@@ -91,6 +88,7 @@ struct _HTMLEngine {
 
 	gboolean editable;
 	GList *cut_buffer;
+	GList *cut_buffer_stack;
 
 	/* Freeze counter.  When greater than zero, we never trigger relayouts
            nor repaints.  When going from nonzero to zero, we relayout and
@@ -147,9 +145,6 @@ struct _HTMLEngine {
 	/* timer id for parsing routine */
 	guint timerId;
 
-	/* Should the background be painted? */
-	gboolean bDrawBackground;
-
 	/* FIXME: replace with a `gchar *'?  */
 	GString *title;
 
@@ -227,16 +222,24 @@ struct _HTMLEngine {
 
 	/* Font style for insertion.  If HTML_FONT_STYLE_DEFAULT, use that of
            the text we are in.  */
-	GtkHTMLFontStyle insertion_font_style;
+	GtkHTMLFontStyle  insertion_font_style;
+	HTMLColor        *insertion_color;
+	gchar            *insertion_url;
+	gchar            *insertion_target;
 
 	/* This is set to TRUE when at least one element is selected (in whole
            or in part), to FALSE when no item is selected at all.  */
 	gboolean active_selection;
+	gboolean shift_selection;
 
 	/* This object is used to update the keyboard selection in the
            idle loop.  */
 	HTMLEngineEditSelectionUpdater *selection_updater;
 
+	/* keeps selection between operation, which are changing selection
+	   and need restore it after done */
+	GList *selection_stack;
+
 	/* search & replace */
 	HTMLSearch  *search_info;
 	HTMLReplace *replace_info;
@@ -262,7 +265,7 @@ struct _HTMLEngineClass {
 
 /* Object construction.  */
 guint       html_engine_get_type      (void);
-HTMLEngine *html_engine_new           (void);
+HTMLEngine *html_engine_new           (GtkWidget *);
 void        html_engine_realize       (HTMLEngine *engine,
 				       GdkWindow  *window);
 
@@ -343,6 +346,10 @@ void   html_engine_unselect_all          (HTMLEngine *e,
 void   html_engine_disable_selection     (HTMLEngine *e);
 gchar *html_engine_get_selection_string  (HTMLEngine *e);
 
+/* Cursor normalization.  */
+
+void  html_engine_normalize_cursor  (HTMLEngine *e);
+
 /* Freezing/thawing.  */
 gboolean  html_engine_frozen  (HTMLEngine *engine);
 void      html_engine_freeze  (HTMLEngine *engine);
@@ -370,4 +377,7 @@ void      html_engine_replace                   (HTMLEngine *e,
 void      html_engine_replace_do                (HTMLEngine *e, HTMLReplaceQueryAnswer answer);
 gint      html_engine_replaced                  (void);
 
+/* Magic links */
+void      html_engine_init_magic_links          (void);
+
 #endif /* _HTMLENGINE_H_ */
