@@ -19,10 +19,12 @@
    Boston, MA 02111-1307, USA.
 */
 #include <gtk/gtk.h>
+#include <string.h>
 #include "htmliframe.h"
 
 HTMLIFrameClass html_iframe_class;
 static HTMLEmbeddedClass *parent_class = NULL;
+static gboolean calc_size (HTMLObject *o, HTMLPainter *painter);
 
 static void
 iframe_url_requested (GtkHTML *html, const char *url, GtkHTMLStream *handle, gpointer data)
@@ -31,7 +33,7 @@ iframe_url_requested (GtkHTML *html, const char *url, GtkHTMLStream *handle, gpo
 	GtkHTML *parent = GTK_HTML (HTML_EMBEDDED(iframe)->parent);
 	char *new_url = NULL;
 
-	g_warning (" this is a note");
+	/* FIXME this is not exactly the single safest method of expanding a relative url */
 	if (!strstr (url, ":"))
 		new_url = g_strconcat (iframe->url, url, NULL);
 	
@@ -40,12 +42,6 @@ iframe_url_requested (GtkHTML *html, const char *url, GtkHTMLStream *handle, gpo
 	
 	if (new_url)
 		g_free (new_url);
-
-	html_embedded_size_recalc(HTML_EMBEDDED (iframe));
-	/*
-	  html_engine_schedule_update(GTK_HTML (HTML_EMBEDDED(iframe)->widget)->engine); */
-	html_engine_schedule_update(parent->engine);
-	
 }
 
 static void
@@ -66,6 +62,15 @@ iframe_link_clicked (GtkHTML *html, const gchar *url, gpointer data)
 	gtk_signal_emit_by_name (GTK_OBJECT (parent), "link_clicked", url);
 }
 
+static void
+iframe_size_changed (GtkAdjustment *adj, gpointer data)
+{
+	HTMLIFrame *iframe = HTML_IFRAME (data);
+	GtkHTML *parent = GTK_HTML (HTML_EMBEDDED(iframe)->parent);
+	
+	html_engine_schedule_update (parent->engine);
+}
+	
 HTMLObject *
 html_iframe_new (GtkWidget *parent, 
 		 char *src, 
@@ -136,12 +141,13 @@ calc_size (HTMLObject *o,
 	   HTMLPainter *painter)
 {
 	HTMLIFrame *iframe;
-	guint pixel_size;
+	guint pixel_size = 1;
 	gint width, height;
 	gint old_width, old_ascent, old_descent;
 	GtkHTML *html;
 	
-	pixel_size = html_painter_get_pixel_size (painter);
+	if (painter)
+		pixel_size = html_painter_get_pixel_size (painter);
        
 	old_width = o->width;
 	old_ascent = o->ascent;
@@ -226,7 +232,9 @@ html_iframe_init (HTMLIFrame *iframe,
 	gtk_signal_connect (GTK_OBJECT (html), "link_clicked",
 			    GTK_SIGNAL_FUNC (iframe_link_clicked),
 			    (gpointer)iframe);	
-
+	gtk_signal_connect (GTK_OBJECT (html), "size_changed",
+			    GTK_SIGNAL_FUNC (iframe_size_changed),
+			    (gpointer)iframe);	
 	/*
 	  gtk_signal_connect (GTK_OBJECT (html), "button_press_event",
 	  GTK_SIGNAL_FUNC (iframe_button_press_event), iframe);
@@ -234,8 +242,21 @@ html_iframe_init (HTMLIFrame *iframe,
 	gtk_signal_emit_by_name (GTK_OBJECT (GTK_HTML (html)->engine), 
 				 "url_requested", src, handle);
 
-	gtk_widget_set_usize (scrolled_window, width, height);
+	if ((width > 0) && (height > 0))
+		gtk_widget_set_usize (scrolled_window, width, height);
 
+	/* 
+	   FIXME: huge hack to get the widget to allocate it's size 
+	   before it is actually drawn.  Place it in the gtklayout but offscreen
+	   it will be moved to the proper place in the draw routine, but it's size
+	   allocation needs to happen now.
+	*/
+	gtk_layout_put(GTK_LAYOUT(parent_html), scrolled_window,
+		       10000, 10000);
+	HTML_EMBEDDED (iframe)->abs_x = 10000;
+	HTML_EMBEDDED (iframe)->abs_y = 10000;
+	gtk_widget_show (scrolled_window);			
+	
 	iframe->scroll = scrolled_window;
 
 	html_embedded_set_widget (em, scrolled_window);	
@@ -244,6 +265,11 @@ html_iframe_init (HTMLIFrame *iframe,
 	gtk_signal_connect(GTK_OBJECT(scrolled_window), "button_press_event",
 			   GTK_SIGNAL_FUNC(html_iframe_grab_cursor), NULL);
 
+	/* inherit the current colors from our parent */
+	html_colorset_set_unchanged (GTK_HTML (html)->engine->defaultSettings->color_set,
+				     parent_html->engine->settings->color_set);
+	html_colorset_set_unchanged (GTK_HTML (html)->engine->settings->color_set,
+				     parent_html->engine->settings->color_set);
 	/*
 	gtk_signal_connect (GTK_OBJECT (html), "title_changed",
 			    GTK_SIGNAL_FUNC (title_changed_cb), (gpointer)app);
@@ -288,4 +314,5 @@ html_iframe_class_init (HTMLIFrameClass *klass,
 	object_class->calc_min_width = calc_min_width;
 	object_class->reset = reset;
 	object_class->set_max_width = set_max_width;
+	
 }
