@@ -54,11 +54,6 @@ struct _HTMLImageFactory {
 
 #define DEFAULT_SIZE 48
 
-#if 1
-#define gdk_pixbuf_animation_get_width animation_actual_width
-#define gdk_pixbuf_animation_get_height animation_actual_height
-#endif
-
 
 HTMLImageClass html_image_class;
 static HTMLObjectClass *parent_class = NULL;
@@ -71,45 +66,6 @@ static void                html_image_pointer_destroy   (HTMLImagePointer *ip);
 static void render_cur_frame (HTMLImage *image, gint nx, gint ny, const GdkColor *highlight_color);
 
 
-/* FIXME these will be replaced with the proper functions from gdk-pixbuf soon */
-static gint
-animation_actual_height (GdkPixbufAnimation *ganim) 
-{
-	GdkPixbufFrame    *frame;
-	GList *cur = gdk_pixbuf_animation_get_frames (ganim);
-	gint h = 0, nh;
-
-	while (cur) {
-		frame = (GdkPixbufFrame *) cur->data;
-
-		nh = gdk_pixbuf_get_height (gdk_pixbuf_frame_get_pixbuf (frame))
-			+ gdk_pixbuf_frame_get_y_offset (frame);
-		h = MAX (h, nh);
-		cur = cur->next;
-	}
-
-	return h;
-}
-
-static gint
-animation_actual_width (GdkPixbufAnimation *ganim) 
-{
-	GdkPixbufFrame    *frame;
-	GList *cur = gdk_pixbuf_animation_get_frames (ganim);
-	gint w = 0, nw;
-
-	while (cur) {
-		frame = (GdkPixbufFrame *) cur->data;
-
-		nw = gdk_pixbuf_get_width (gdk_pixbuf_frame_get_pixbuf (frame))
-			+ gdk_pixbuf_frame_get_x_offset (frame);
-		w = MAX (w, nw);
-		cur = cur->next;
-	}
-
-	return w;
-}
-
 static guint
 get_actual_width (HTMLImage *image,
 		  HTMLPainter *painter)
@@ -197,20 +153,12 @@ destroy (HTMLObject *o)
 	g_free (image->url);
 	g_free (image->target);
 
+	if (image->color)
+		html_color_unref (image->color);
+
 	HTML_OBJECT_CLASS (parent_class)->destroy (o);
 }
 
-static void
-reset (HTMLObject *self)
-{
-	HTMLImage *image;
-	
-	image = HTML_IMAGE (self);
-	image->color_allocated = FALSE;
-
-	(* HTML_OBJECT_CLASS (parent_class)->reset) (self);
-}
-
 static void
 copy (HTMLObject *self,
       HTMLObject *dest)
@@ -229,8 +177,9 @@ copy (HTMLObject *self,
 	dimg->url = g_strdup (simg->url);
 	dimg->target = g_strdup (simg->target);
 	dimg->color = simg->color;
+	if (simg->color)
+		html_color_ref (dimg->color);
 	dimg->have_color = simg->have_color;
-	dimg->color_allocated = FALSE;
 	dimg->valign = simg->valign;
 	dimg->animation = NULL;          /* don't bother with animation copying now. TODO */
 	dimg->hspace = simg->hspace;
@@ -337,7 +286,7 @@ draw (HTMLObject *o,
 		hspace = image->hspace * pixel_size;
 		vspace = image->vspace * pixel_size;
 
-		html_painter_draw_panel (painter, 
+		html_painter_draw_panel (painter,
 					 o->x + tx + hspace,
 					 o->y + ty - o->ascent + vspace,
 					 o->width - 2 * hspace,
@@ -353,17 +302,14 @@ draw (HTMLObject *o,
 	scale_height = get_actual_height (image, painter);
 
 	if (o->selected)
-		highlight_color = html_painter_get_default_highlight_color (painter);
+		highlight_color = &html_colorset_get_color (painter->color_set, HTMLHighlightColor)->color;
 	else
 		highlight_color = NULL;
 
 	if (image->border) {
 		if (image->have_color) {
-			if (!image->color_allocated) {
-				html_painter_alloc_color (painter, &image->color);
-				image->color_allocated = TRUE;
-			}
-			html_painter_set_pen (painter, &image->color);
+			html_color_alloc (image->color, painter);
+			html_painter_set_pen (painter, &image->color->color);
 		}
 		
 		html_painter_draw_panel (painter, 
@@ -466,6 +412,34 @@ get_target (HTMLObject *o)
 	return image->target;
 }
 
+static HTMLObject *
+set_link (HTMLObject *self, HTMLColor *color, const gchar *url, const gchar *target)
+{
+	HTMLImage *image = HTML_IMAGE (self);
+
+	g_free (image->url);
+	g_free (image->target);
+
+	image->url = g_strdup (url);
+	image->target = g_strdup (target);
+
+	return NULL;
+}
+
+static HTMLObject *
+remove_link (HTMLObject *self, HTMLColor *color)
+{
+	HTMLImage *image = HTML_IMAGE (self);
+
+	g_free (image->url);
+	g_free (image->target);
+
+	image->url = g_strdup ("");
+	image->target = g_strdup ("");
+
+	return NULL;
+}
+
 static gboolean
 accepts_cursor (HTMLObject *o)
 {
@@ -501,7 +475,6 @@ html_image_class_init (HTMLImageClass *image_class,
 	html_object_class_init (object_class, type, size);
 
 	object_class->copy = copy;
-	object_class->reset = reset;
 	object_class->draw = draw;	
 	object_class->destroy = destroy;
 	object_class->calc_min_width = calc_min_width;
@@ -509,6 +482,8 @@ html_image_class_init (HTMLImageClass *image_class,
 	object_class->calc_size = calc_size;
 	object_class->get_url = get_url;
 	object_class->get_target = get_target;
+	object_class->set_link = set_link;
+	object_class->remove_link = remove_link;
 	object_class->accepts_cursor = accepts_cursor;
 	object_class->get_valign = get_valign;
 	object_class->save = save;
@@ -525,7 +500,7 @@ html_image_init (HTMLImage *image,
 		 const gchar *target,
 		 gint16 width, gint16 height,
 		 gint8 percent, gint8 border,
-		 const GdkColor *color,
+		 HTMLColor *color,
 		 HTMLVAlignType valign)
 {
 	HTMLObject *object;
@@ -536,17 +511,20 @@ html_image_init (HTMLImage *image,
 
 	html_object_init (object, HTML_OBJECT_CLASS (klass));
 
-	image->url = g_strdup (url);
-	image->target = g_strdup (target);
+	image->url = g_strdup ((url) ? url : "");
+	image->target = g_strdup ((url) ? target : "");
 
 	image->specified_width = width;
 	image->specified_height = height;
 	image->border = border;
 
-	image->color_allocated = FALSE;
 	if (color) {
-		image->color = *color;
+		image->color = color;
 		image->have_color = TRUE;
+		html_color_ref (color);
+	} else {
+		image->color = NULL;
+		image->have_color = FALSE;
 	}
 
 	image->animation = NULL;
@@ -570,7 +548,7 @@ html_image_new (HTMLImageFactory *imf,
 		const gchar *target,
 		gint16 width, gint16 height,
 		gint8 percent, gint8 border,	
-		const GdkColor *color,
+		HTMLColor *color,
 		HTMLVAlignType valign)
 {
 	HTMLImage *image;
