@@ -39,8 +39,8 @@ destroy (HTMLObject *object)
 	HTMLLinkText *link_text;
 
 	link_text = HTML_LINK_TEXT (object);
-	g_free (link_text->url);
-	g_free (link_text->target);
+	if (link_text->url)    g_free (link_text->url);
+	if (link_text->target) g_free (link_text->target);
 
 	(* HTML_OBJECT_CLASS (parent_class)->destroy) (object);
 }
@@ -67,6 +67,22 @@ get_target (HTMLObject *object)
 	return HTML_LINK_TEXT (object)->target;
 }
 
+static HTMLObject *
+set_link (HTMLObject *self, HTMLColor *color, const gchar *url, const gchar *target)
+{
+	HTMLLinkText *link = HTML_LINK_TEXT (self);
+
+	g_free (link->url);
+	g_free (link->target);
+
+	link->url = g_strdup (url);
+	link->target = g_strdup (target);
+
+	html_text_set_color (HTML_TEXT (self), NULL, color);
+
+	return NULL;
+}
+
 static HTMLText *
 split (HTMLText *self,
        guint offset)
@@ -81,7 +97,7 @@ split (HTMLText *self,
 
 	new = HTML_TEXT (html_link_text_new (self->text + offset,
 					     self->font_style,
-					     &self->color,
+					     self->color,
 					     link_text->url,
 					     link_text->target));
 
@@ -108,7 +124,7 @@ save (HTMLObject *self,
       HTMLEngineSaveState *state)
 {
 	if (! html_engine_save_output_string (state, "<A HREF=\"")
-	    || ! html_engine_save_output_string (state, HTML_LINK_TEXT (self)->url)
+	    || ! html_engine_save_output_string (state, "%s", HTML_LINK_TEXT (self)->url)
 	    || ! html_engine_save_output_string (state, "\">"))
 		return FALSE;
 
@@ -145,6 +161,7 @@ html_link_text_class_init (HTMLLinkTextClass *klass,
 	object_class->copy = copy;
 	object_class->get_url = get_url;
 	object_class->get_target = get_target;
+	object_class->set_link = set_link;
 	object_class->save = save;
 
 	text_class->split = split;
@@ -158,7 +175,7 @@ html_link_text_init (HTMLLinkText *link_text_object,
 		     HTMLLinkTextClass *klass,
 		     const gchar *text,
 		     GtkHTMLFontStyle font_style,
-		     const GdkColor *color,
+		     HTMLColor *color,
 		     const gchar *url,
 		     const gchar *target)
 {
@@ -174,7 +191,7 @@ html_link_text_init (HTMLLinkText *link_text_object,
 HTMLObject *
 html_link_text_new (const gchar *text,
 		    GtkHTMLFontStyle font_style,
-		    const GdkColor *color,
+		    HTMLColor *color,
 		    const gchar *url,
 		    const gchar *target)
 {
