@@ -23,6 +23,7 @@
 
 #include "htmltext.h"
 #include "htmllinktextmaster.h"
+#include "htmlengine-edit-paste.h"
 
 
 HTMLLinkTextMasterClass html_link_text_master_class;
@@ -44,14 +45,33 @@ destroy (HTMLObject *object)
 	(* HTML_OBJECT_CLASS (parent_class)->destroy) (object);
 }
 
+static void
+copy_helper (HTMLText *self,
+	     HTMLText *dest)
+{
+	HTML_LINK_TEXT_MASTER (dest)->url = g_strdup (HTML_LINK_TEXT_MASTER (self)->url);
+	HTML_LINK_TEXT_MASTER (dest)->target = g_strdup (HTML_LINK_TEXT_MASTER (self)->target);
+}
+
 static void
 copy (HTMLObject *self,
       HTMLObject *dest)
 {
 	(* HTML_OBJECT_CLASS (parent_class)->copy) (self, dest);
+	copy_helper (HTML_TEXT (self), HTML_TEXT (dest));
+}
 
-	HTML_LINK_TEXT_MASTER (dest)->url = g_strdup (HTML_LINK_TEXT_MASTER (self)->url);
-	HTML_LINK_TEXT_MASTER (dest)->target = g_strdup (HTML_LINK_TEXT_MASTER (self)->target);
+static HTMLText *
+extract_text (HTMLText *text,
+	      guint offset,
+	      gint len)
+{
+	HTMLText *new;
+
+	new = (* HTML_TEXT_CLASS (parent_class)->extract_text) (text, offset, len);
+	copy_helper (text, new);
+
+	return new;
 }
 
 static const gchar *
@@ -66,6 +86,15 @@ get_target (HTMLObject *object)
 	return HTML_LINK_TEXT_MASTER (object)->target;
 }
 
+static HTMLObject *
+remove_link (HTMLObject *self, HTMLColor *color)
+{
+	HTMLText *text = HTML_TEXT (self);
+
+	return html_text_master_new_with_len (text->text, text->text_len, text->font_style, color);
+}
+
+
 static HTMLText *
 split (HTMLText *self,
        guint offset)
@@ -76,9 +105,7 @@ split (HTMLText *self,
 	if (new == NULL)
 		return NULL;
 
-	HTML_LINK_TEXT_MASTER (new)->url = g_strdup (HTML_LINK_TEXT_MASTER (self)->url);
-	HTML_LINK_TEXT_MASTER (new)->target = g_strdup (HTML_LINK_TEXT_MASTER (self)->target);
-
+	copy_helper (self, new);
 	return new;
 }
 
@@ -98,7 +125,7 @@ save (HTMLObject *self,
       HTMLEngineSaveState *state)
 {
 	if (! html_engine_save_output_string (state, "<A HREF=\"")
-	    || ! html_engine_save_output_string (state, HTML_LINK_TEXT_MASTER (self)->url)
+	    || ! html_engine_save_output_string (state, "%s", HTML_LINK_TEXT_MASTER (self)->url)
 	    || ! html_engine_save_output_string (state, "\">"))
 		return FALSE;
 
@@ -127,7 +154,7 @@ get_selection (HTMLObject *self,
 	new = html_link_text_master_new_with_len (HTML_TEXT(self)->text + select_start,
 						  select_length,
 						  HTML_TEXT (self)->font_style,
-						  & HTML_TEXT (self)->color,
+						  HTML_TEXT (self)->color,
 						  HTML_LINK_TEXT_MASTER (self)->url,
 						  HTML_LINK_TEXT_MASTER (self)->target);
 
@@ -165,12 +192,13 @@ html_link_text_master_class_init (HTMLLinkTextMasterClass *klass,
 	object_class->copy = copy;
 	object_class->get_url = get_url;
 	object_class->get_target = get_target;
+	object_class->remove_link = remove_link;	
 	object_class->save = save;
 	object_class->get_selection = get_selection;
 
 	text_class->split = split;
 	text_class->get_font_style = get_font_style;
-
+	text_class->extract_text = extract_text;
 	parent_class = &html_text_master_class;
 }
 
@@ -180,7 +208,7 @@ html_link_text_master_init (HTMLLinkTextMaster *link_text_master_object,
 			    const gchar *text,
 			    gint len,
 			    GtkHTMLFontStyle font_style,
-			    const GdkColor *color,
+			    HTMLColor *color,
 			    const gchar *url,
 			    const gchar *target)
 {
@@ -202,7 +230,7 @@ HTMLObject *
 html_link_text_master_new_with_len (const gchar *text,
 				    gint len,
 				    GtkHTMLFontStyle font_style,
-				    const GdkColor *color,
+				    HTMLColor *color,
 				    const gchar *url,
 				    const gchar *target)
 {
@@ -227,7 +255,7 @@ html_link_text_master_new_with_len (const gchar *text,
 HTMLObject *
 html_link_text_master_new (const gchar *text,
 			   GtkHTMLFontStyle font_style,
-			   const GdkColor *color,
+			   HTMLColor *color,
 			   const gchar *url,
 			   const gchar *target)
 {
@@ -240,3 +268,17 @@ html_link_text_master_set_url (HTMLLinkTextMaster *link, const gchar *url)
 	g_free (link->url);
 	link->url = g_strdup (url);
 }
+
+void
+html_link_text_master_to_text (HTMLLinkTextMaster *link, HTMLEngine *e)
+{
+	HTMLObject *new_text;
+
+	new_text = html_text_master_new (HTML_TEXT (link)->text,
+					 GTK_HTML_FONT_STYLE_DEFAULT,
+					 html_colorset_get_color (e->settings->color_set, HTMLTextColor));
+
+	html_engine_replace_by_object (e, HTML_OBJECT (link), 0,
+				       HTML_OBJECT (link), HTML_TEXT (link)->text_len,
+				       new_text);
+}
