@@ -351,6 +351,12 @@ get_cursor_base (HTMLObject *self,
 		*x += self->width;
 }
 
+static guint
+get_length (HTMLObject *self)
+{
+	return 1;
+}
+
 static gboolean
 select_range (HTMLObject *self,
 	      HTMLEngine *engine,
@@ -361,7 +367,7 @@ select_range (HTMLObject *self,
 	gboolean selected;
 	gboolean changed;
 
-	if (length != 0)
+	if (length > 0 || (length == -1 && start < html_object_get_length (self)))
 		selected = TRUE;
 	else
 		selected = FALSE;
@@ -487,6 +493,7 @@ html_object_class_init (HTMLObjectClass *klass,
 	klass->get_url = get_url;
 	klass->get_target = get_target;
 	klass->find_anchor = find_anchor;
+	klass->set_link = NULL;
 	klass->set_bg_color = set_bg_color;
 	klass->check_point = check_point;
 	klass->relayout = relayout;
@@ -504,6 +511,7 @@ html_object_class_init (HTMLObjectClass *klass,
 	klass->check_page_split = check_page_split;
 	klass->search = search;
 	klass->search_next = search;
+	klass->get_length = get_length;
 }
 
 void
@@ -945,3 +953,25 @@ html_object_search_next (HTMLObject *self, HTMLSearch *info)
 {
 	return (* HO_CLASS (self)->search_next) (self, info);
 }
+
+HTMLObject *
+html_object_set_link (HTMLObject *self,
+		      HTMLColor *color,
+		      const gchar *url,
+		      const gchar *target)
+{
+	return (HO_CLASS (self)->set_link) ? (* HO_CLASS (self)->set_link) (self, color, url, target) : NULL;
+}
+
+HTMLObject *
+html_object_remove_link (HTMLObject *self,
+			 HTMLColor *color)
+{
+	return (HO_CLASS (self)->remove_link) ? (* HO_CLASS (self)->remove_link) (self, color) : NULL;
+}
+
+guint
+html_object_get_length (HTMLObject *self)
+{
+	return (* HO_CLASS (self)->get_length) (self);
+}
