@@ -1330,13 +1330,10 @@ search (HTMLObject *obj, HTMLSearch *info)
 	guint r, c, i, j;
 	gboolean next = FALSE;
 
-	printf ("search table\n");
-
 	/* search_next? */
 	if (html_search_child_on_stack (info, obj)) {
 		cur  = html_search_pop (info);
 		next = TRUE;
-		printf ("search table next\n");
 	}
 
 	if (info->forward) {
@@ -1359,8 +1356,6 @@ search (HTMLObject *obj, HTMLSearch *info)
 					continue;
 				}
 
-				printf ("search table cell %d,%d %p\n", r, c, cell);
-
 				html_search_push (info, HTML_OBJECT (cell));
 				if (html_object_search (HTML_OBJECT (cell), info)) {
 					return TRUE;
@@ -1388,8 +1383,6 @@ search (HTMLObject *obj, HTMLSearch *info)
 					continue;
 				}
 
-				printf ("search backward table cell %d,%d %p\n", r, c, cell);
-
 				html_search_push (info, HTML_OBJECT (cell));
 				if (html_object_search (HTML_OBJECT (cell), info)) {
 					return TRUE;
