@@ -38,6 +38,7 @@ static HTMLObjectClass *parent_class = NULL;
 
 static void
 get_tags (const HTMLText *text,
+	  const HTMLEngineSaveState *state,
 	  gchar *opening_tags,
 	  gchar *ending_tags)
 {
@@ -55,6 +56,15 @@ get_tags (const HTMLText *text,
 	opening_p = opening_tags;
 	ending_p = ending_tags;
 
+	if (!html_color_equal (text->color, html_colorset_get_color (state->engine->settings->color_set, HTMLTextColor))
+	    && !html_color_equal (text->color, html_colorset_get_color (state->engine->settings->color_set, HTMLLinkColor))) {
+		opening_p += sprintf (opening_p, "<FONT COLOR=#%02x%02x%02x>",
+				      text->color->color.red   >> 8,
+				      text->color->color.green >> 8,
+				      text->color->color.blue  >> 8);
+		ending_p += sprintf (ending_p, "</FONT>");
+	}
+
 	size = font_style & GTK_HTML_FONT_STYLE_SIZE_MASK;
 	if (size != 0) {
 		opening_p += sprintf (opening_p, "<FONT SIZE=%d>", size);
@@ -120,23 +130,12 @@ copy_helper (HTMLText *src,
 
 	dest->font_style = src->font_style;
 	dest->color = src->color;
-	dest->color_allocated = src->color_allocated;
+	html_color_ref (dest->color);
 }
 
 
 /* HTMLObject methods.  */
 
-static void
-reset (HTMLObject *self)
-{
-	HTMLText *text;
-
-	text = HTML_TEXT (self);
-	text->color_allocated = FALSE;
-
-	(* HTML_OBJECT_CLASS (parent_class)->reset) (self);
-}
-
 static void
 copy (HTMLObject *self,
       HTMLObject *dest)
@@ -190,18 +189,17 @@ draw (HTMLObject *o,
       gint tx, gint ty)
 {
 	GtkHTMLFontStyle font_style;
-	HTMLText *htmltext;
-
-	htmltext = HTML_TEXT (o);
+	HTMLText *text = HTML_TEXT (o);
 
 	if (y + height < o->y - o->ascent || y > o->y + o->descent)
 		return;
 
-	font_style = html_text_get_font_style (htmltext);
+	font_style = html_text_get_font_style (text);
 	html_painter_set_font_style (p, font_style);
 
-	html_painter_set_pen (p, html_text_get_color (htmltext, p));
-	html_painter_draw_text (p, o->x + tx, o->y + ty, htmltext->text, -1);
+	html_color_alloc (text->color, p);
+	html_painter_set_pen (p, &text->color->color);
+	html_painter_draw_text (p, o->x + tx, o->y + ty, text->text, -1);
 }
 
 static gboolean
@@ -225,15 +223,15 @@ save (HTMLObject *self,
 
 	text = HTML_TEXT (self);
 
-	get_tags (text, opening_tags, closing_tags);
+	get_tags (text, state, opening_tags, closing_tags);
 
-	if (! html_engine_save_output_string (state, opening_tags))
+	if (! html_engine_save_output_string (state, "%s", opening_tags))
 		return FALSE;
 
 	if (! html_engine_save_encode (state, text->text, text->text_len))
 		return FALSE;
 
-	if (! html_engine_save_output_string (state, closing_tags))
+	if (! html_engine_save_output_string (state, "%s", closing_tags))
 		return FALSE;
 
 	return TRUE;
@@ -244,10 +242,9 @@ save_plain (HTMLObject *self,
       HTMLEngineSaveState *state)
 {
 	HTMLText *text;
-	char *str = NULL;
 	text = HTML_TEXT (self);
 
-	if (! html_engine_save_output_string (state, text->text))
+	if (! html_engine_save_output_string (state, "%s", text->text))
 		return FALSE;
 	
 	return TRUE;
@@ -289,6 +286,12 @@ extract_text (HTMLText *text,
 	return new;
 }
 
+static guint
+get_length (HTMLObject *self)
+{
+	return HTML_TEXT (self)->text_len;
+}
+
 /* #define DEBUG_NBSP */
 
 static void
@@ -460,7 +463,7 @@ split (HTMLText *self,
 
 	new = g_malloc (HTML_OBJECT (self)->klass->object_size);
 	html_text_init (new, HTML_TEXT_CLASS (HTML_OBJECT (self)->klass),
-			self->text + offset, -1, self->font_style, &self->color);
+			self->text + offset, -1, self->font_style, self->color);
 
 	self->text = g_realloc (self->text, offset + 1);
 	self->text[offset] = '\0';
@@ -488,7 +491,7 @@ check_merge (HTMLText *self,
 		return TRUE;
 	if (self->font_style != text->font_style)
 		return FALSE;
-	if (! gdk_color_equal (&self->color, &text->color))
+	if (! html_color_equal (self->color, text->color))
 		return FALSE;
 
 	return TRUE;
@@ -516,14 +519,11 @@ get_font_style (const HTMLText *text)
 	return font_style;
 }
 
-static const GdkColor *
+static HTMLColor *
 get_color (HTMLText *text,
 	   HTMLPainter *painter)
 {
-	if (! text->color_allocated)
-		html_painter_alloc_color (painter, &text->color);
-
-	return &text->color;
+	return text->color;
 }
 
 static void
@@ -547,13 +547,14 @@ set_font_style (HTMLText *text,
 static void
 set_color (HTMLText *text,
 	   HTMLEngine *engine,
-	   const GdkColor *color)
+	   HTMLColor *color)
 {
-	if (gdk_color_equal (&text->color, color))
+	if (html_color_equal (text->color, color))
 		return;
 
-	text->color = *color;
-	text->color_allocated = FALSE;
+	html_color_unref (text->color);
+	html_color_ref (color);
+	text->color = color;
 
 	if (engine != NULL) {
 		html_object_relayout (HTML_OBJECT (text)->parent, engine, HTML_OBJECT (text));
@@ -564,7 +565,9 @@ set_color (HTMLText *text,
 static void
 destroy (HTMLObject *obj)
 {
-	g_free (HTML_TEXT (obj)->text);
+	HTMLText *text = HTML_TEXT (obj);
+	html_color_unref (text->color);
+	g_free (text->text);
 	HTML_OBJECT_CLASS (parent_class)->destroy (obj);
 }
 
@@ -587,7 +590,6 @@ html_text_class_init (HTMLTextClass *klass,
 	html_object_class_init (object_class, type, object_size);
 
 	object_class->destroy = destroy;
-	object_class->reset = reset;
 	object_class->copy = copy;
 	object_class->draw = draw;
 	object_class->accepts_cursor = accepts_cursor;
@@ -596,6 +598,7 @@ html_text_class_init (HTMLTextClass *klass,
 	object_class->get_cursor_base = get_cursor_base;
 	object_class->save = save;
 	object_class->save_plain = save_plain;
+	object_class->get_length = get_length;
 
 	/* HTMLText methods.  */
 
@@ -620,10 +623,12 @@ html_text_init (HTMLText *text_object,
 		const gchar *text,
 		gint len,
 		GtkHTMLFontStyle font_style,
-		const GdkColor *color)
+		HTMLColor *color)
 {
 	HTMLObject *object;
 
+	g_assert (color);
+
 	object = HTML_OBJECT (text_object);
 
 	html_object_init (object, HTML_OBJECT_CLASS (klass));
@@ -640,23 +645,15 @@ html_text_init (HTMLText *text_object,
 	convert_nbsp (text_object->text, text_object->text_len);
 
 	text_object->font_style = font_style;
-
-	if (color != NULL) {
-		text_object->color = *color;
-	} else {
-		text_object->color.red = 0;
-		text_object->color.green = 0;
-		text_object->color.blue = 0;
-	}
-
-	text_object->color_allocated = FALSE;
+	html_color_ref (color);
+	text_object->color = color;
 }
 
 HTMLObject *
 html_text_new_with_len (const gchar *text,
 			gint len,
 			GtkHTMLFontStyle font,
-			const GdkColor *color)
+			HTMLColor *color)
 {
 	HTMLText *text_object;
 
@@ -670,7 +667,7 @@ html_text_new_with_len (const gchar *text,
 HTMLObject *
 html_text_new (const gchar *text,
 	       GtkHTMLFontStyle font,
-	       const GdkColor *color)
+	       HTMLColor *color)
 {
 	return html_text_new_with_len (text, -1, font, color);
 }
@@ -768,7 +765,7 @@ html_text_get_font_style (const HTMLText *text)
 	return (* HT_CLASS (text)->get_font_style) (text);
 }
 
-const GdkColor *
+HTMLColor *
 html_text_get_color (HTMLText *text,
 		     HTMLPainter *painter)
 {
@@ -791,7 +788,7 @@ html_text_set_font_style (HTMLText *text,
 void
 html_text_set_color (HTMLText *text,
 		     HTMLEngine *engine,
-		     const GdkColor *color)
+		     HTMLColor *color)
 {
 	g_return_if_fail (text != NULL);
 	g_return_if_fail (color != NULL);
