@@ -26,7 +26,14 @@
 #include "htmlclueflow.h"
 #include "htmltextmaster.h"
 #include "htmltextslave.h"
+#include "htmlentity.h"
 #include <string.h>
+#include <sys/types.h>
+#include <regex.h>
+#include "htmlengine.h"
+#include "htmlengine-edit.h"
+#include "htmlengine-edit-paste.h"
+#include "htmllinktextmaster.h"
 
 
 HTMLTextMasterClass html_text_master_class;
@@ -187,6 +194,7 @@ check_point (HTMLObject *self,
 	for (p = self->next;
 	     p != NULL && HTML_OBJECT_TYPE (p) == HTML_TYPE_TEXTSLAVE;
 	     p = p->next) {
+
 		if (y > p->y + p->descent)
 			continue;
 
@@ -321,7 +329,7 @@ get_selection (HTMLObject *self,
 	text_self = HTML_TEXT (self);
 
 	new = html_text_master_new_with_len (text_self->text + select_start,
-					     select_length, text_self->font_style, & text_self->color);
+					     select_length, text_self->font_style, text_self->color);
 
 	if (size_return != NULL)
 		*size_return = select_length;
@@ -329,6 +337,14 @@ get_selection (HTMLObject *self,
 	return new;
 }
 
+static HTMLObject *
+set_link (HTMLObject *self, HTMLColor *color, const gchar *url, const gchar *target)
+{
+	HTMLText *text = HTML_TEXT (self);
+
+	return html_link_text_master_new_with_len (text->text, text->text_len, text->font_style, color, url, target);
+}
+
 static void
 append_selection_string (HTMLObject *self,
 			 GString *buffer)
@@ -548,8 +564,9 @@ merge (HTMLText *self,
 
 	if (self->text_len == 0) {
 		self->font_style = other->font_style;
+		html_color_unref (self->color);
 		self->color = other->color;
-		self->color_allocated = other->color_allocated;
+		html_color_ref (self->color);
 	}
 
 	if (prepend)
@@ -600,6 +617,7 @@ insert_text (HTMLText *text,
 	master = HTML_TEXT_MASTER (text);
 	chars_inserted = HTML_TEXT_CLASS (parent_class)->insert_text (text, engine, offset, p, len);
 
+	/* update selection */
 	if (chars_inserted > 0
 	    && offset >= master->select_start
 	    && offset < master->select_start + master->select_length)
@@ -666,6 +684,7 @@ html_text_master_class_init (HTMLTextMasterClass *klass,
 	object_class->select_range = select_range;
 	object_class->get_selection = get_selection;
 	object_class->append_selection_string = append_selection_string;
+	object_class->set_link = set_link;
 
 	/* HTMLText methods.  */
 
@@ -685,7 +704,7 @@ html_text_master_init (HTMLTextMaster *master,
 		       const gchar *text,
 		       gint len,
 		       GtkHTMLFontStyle font_style,
-		       const GdkColor *color)
+		       HTMLColor *color)
 {
 	HTMLText* html_text;
 	HTMLObject *object;
@@ -703,7 +722,7 @@ HTMLObject *
 html_text_master_new_with_len (const gchar *text,
 			       gint len,
 			       GtkHTMLFontStyle font_style,
-			       const GdkColor *color)
+			       HTMLColor *color)
 {
 	HTMLTextMaster *master;
 
@@ -716,7 +735,7 @@ html_text_master_new_with_len (const gchar *text,
 HTMLObject *
 html_text_master_new (const gchar *text,
 		      GtkHTMLFontStyle font_style,
-		      const GdkColor *color)
+		      HTMLColor *color)
 {
 	return html_text_master_new_with_len (text, -1, font_style, color);
 }
@@ -771,3 +790,90 @@ html_text_master_destroy_slaves (HTMLTextMaster *master)
 		html_object_destroy (p);
 	}
 }
+
+/* magic links */
+
+struct _HTMLMagicInsertMatch
+{
+	gchar *regex;
+	regex_t *preg;
+	gchar *prefix;
+};
+
+typedef struct _HTMLMagicInsertMatch HTMLMagicInsertMatch;
+
+static HTMLMagicInsertMatch mim [] = {
+	{ "(news|telnet|nttp|file|http|ftp|https)://[-a-z0-9.]+[-a-z0-9](:[0-9]*)?(/[-a-z0-9_$.+!*(),;:@&=?/~#]*[^]'.}>) ,\"]*)?", NULL, NULL },
+	{ "www[-a-z0-9.]+[-a-z0-9](:[0-9]*)?(/[-A-Za-z0-9_$.+!*(),;:@&=?/~#]*[^]'.}>) ,\"]*)?", NULL, "http://" },
+	{ "ftp[-a-z0-9.]+[-a-z0-9](:[0-9]*)?(/[-A-Za-z0-9_$.+!*(),;:@&=?/~#]*[^]'.}>) ,\"]*)?", NULL, "ftp://" },
+	{ "[-_a-z0-9.]+@[-_a-z0-9.]+", NULL, "mailto:" }
+};
+
+#define MIM_N (sizeof (mim) / sizeof (mim [0]))
+
+void
+html_engine_init_magic_links (void)
+{
+	gint i;
+
+	for (i=0; i<MIM_N; i++) {
+		mim [i].preg = g_new0 (regex_t, 1);
+		if (regcomp (mim [i].preg, mim [i].regex, REG_EXTENDED | REG_ICASE)) {
+			/* error */
+			g_free (mim [i].preg);
+			mim [i].preg = 0;
+		}
+	}
+}
+
+static void
+paste_link (HTMLEngine *engine, HTMLText *text, gint so, gint eo, gchar *prefix)
+{
+	HTMLObject *new_obj;
+	gchar *href;
+	gchar *base;
+
+	base = g_strndup (text->text + so, eo - so + 1);
+	href = (prefix) ? g_strconcat (prefix, base, NULL) : g_strdup (base);
+	g_free (base);
+
+	new_obj = html_link_text_master_new_with_len
+		(text->text+so,
+		 eo - so + 1,
+		 text->font_style,
+		 html_colorset_get_color (engine->settings->color_set, HTMLLinkColor),
+		 href, NULL);
+	html_engine_replace_by_object (engine,
+				       HTML_OBJECT (text), so, HTML_OBJECT (text), eo + 1,
+				       new_obj);
+
+	g_free (href);
+}
+
+gboolean
+html_text_master_magic_link (HTMLTextMaster *master, HTMLEngine *engine,
+			     guint offset)
+{
+	HTMLText *text = HTML_TEXT (master);
+	regmatch_t pmatch [2];
+	gint i;
+	gchar *prev;
+
+	prev = text->text + offset - 1;
+	while (*prev != ' ' && (guchar)*prev != ENTITY_NBSP && prev > text->text) prev--;
+	if (*prev == ' ' || (guchar) *prev == ENTITY_NBSP)
+		prev++;
+
+	while (prev < text->text+offset) {
+		for (i=0; i<MIM_N; i++) {
+			if (!regexec (mim [i].preg, prev, 2, pmatch, 0)) {
+				gint o = prev - text->text;
+				paste_link (engine, text, pmatch [0].rm_so+o, pmatch [0].rm_eo+o-1, mim [i].prefix);
+				return TRUE;
+			}
+		}
+		prev++;
+	}
+
+	return FALSE;
+}
